type token =
  | ANNOT of (int * string)
  | WDECL of (int * string)
  | IDENTIFIER of (string)
  | CONSTANT of (string)
  | STRING_LITERAL of (string)
  | TYPE_NAME of (string)
  | SIZEOF
  | PTR_OP
  | INC_OP
  | DEC_OP
  | LEFT_OP
  | RIGHT_OP
  | LE_OP
  | GE_OP
  | EQ_OP
  | NE_OP
  | AND_OP
  | OR_OP
  | MUL_ASSIGN
  | DIV_ASSIGN
  | MOD_ASSIGN
  | ADD_ASSIGN
  | SUB_ASSIGN
  | LEFT_ASSIGN
  | RIGHT_ASSIGN
  | AND_ASSIGN
  | XOR_ASSIGN
  | OR_ASSIGN
  | TYPEDEF
  | EXTERN
  | STATIC
  | AUTO
  | REGISTER
  | CHAR
  | SHORT
  | INT
  | LONG
  | SIGNED
  | UNSIGNED
  | FLOAT
  | DOUBLE
  | CONST
  | VOLATILE
  | VOID
  | STRUCT
  | UNION
  | ENUM
  | ELLIPSIS
  | CASE
  | DEFAULT
  | IF
  | ELSE
  | SWITCH
  | WHILE
  | DO
  | FOR
  | GOTO
  | CONTINUE
  | BREAK
  | RETURN
  | SEMICOLON
  | LBRACE
  | RBRACE
  | COMMA
  | COLON
  | EQUAL
  | LPAR
  | RPAR
  | LSQUARE
  | RSQUARE
  | DOT
  | AMP
  | EXL
  | TILDE
  | MINUS
  | PLUS
  | STAR
  | SLASH
  | PERCENT
  | LT
  | GT
  | HAT
  | PIPE
  | QUESTION
  | EOF

open Parsing;;
# 20 "c/cparser.mly"

  open Logic
  open Ptree
  open Cast
  open Parsing

  let loc () = (symbol_start (), symbol_end ())
  let loc_i i = (rhs_start i, rhs_end i)

  let locate x = { node = x; loc = loc() }
  let locate_i i x = { node = x; loc = loc_i i }
  let with_loc l x = { node = x; loc = l }

  let error s = raise (Stdpp.Exc_located (loc (), Stream.Error s))

  let uns () =
    raise (Stdpp.Exc_located (loc (), Stream.Error "Unsupported C syntax"))
  let unss s =
    raise (Stdpp.Exc_located (loc (), 
			      Stream.Error ("Unsupported C syntax: " ^ s)))
  let warning s =
    Format.eprintf "%a warning: %s" Loc.report_line (symbol_start ()) s

  let add_pre_loc lb = function
    | Some (b,_) -> Loc.join (b,0) lb 
    | _ -> lb

  let expr_of_statement s = match s.node with
    | CSnop -> { node = CEnop; loc = s.loc }
    | CSexpr e -> e
    | _ -> assert false

  (* used only for parsing types *)

  type specifier = 
    | Stypedef 
    | Sstorage of storage_class
    | Stype of ctype_expr
    | Slong
    | Sshort
    | Sconst
    | Svolatile
    | Ssigned of bool (* true = signed / false = unsigned *)
    | Sstruct_decl of string option * parameters 
    | Sunion_decl of string option * parameters
	
  and specifiers = specifier list

  and declarator =
    | Dsimple
    | Dpointer of declarator
    | Darray of declarator * cexpr option
    | Dfunction of declarator * parameters * annot option
    | Dbitfield of declarator * cexpr

  and parameters = (specifiers * declarator * string) list

  (* interps a list of specifiers / declarators as a [ctype] *)
  (* TODO: short/long *)

  let storage_class = 
    let rec loop st = function
      | [] -> 
	  st
      | Sstorage st' :: s when st = No_storage -> 
	  loop st' s
      | Sstorage st' :: s when st' = st ->
	  warning "duplicate storage class"; loop st s
      | Sstorage st' :: _ ->
	  error "multiple storage class"
      | _ :: s -> 
	  loop st s
    in
    loop No_storage

  let sign =
    let rec loop so = function
      | [] -> 
	  so
      | Ssigned b' :: sp -> 
	  (match so with 
	     | None -> loop (Some b') sp
	     | Some b when b = b' -> warning "duplicate (un)signed"; loop so sp
	     | Some b -> error "both signed and unsigned")
      | _ :: sp -> 
	  loop so sp
    in
    loop None

  let apply_sign sg ty = match sg, ty with
    | (None | Some false), CTchar -> false
    | None, _ -> true
    | Some b, (CTshort | CTint | CTlong | CTlonglong) -> b
    | Some _, _ -> error "signed or unsigned invalid"

  type length = Short | Long | LongLong

  let length =
    let rec loop lo = function
      | [] -> 
	  lo
      | (Sshort | Slong as s) :: sp -> 
	  (match s, lo with 
	     | Sshort, None -> 
		 loop (Some Short) sp
	     | Slong, None -> 
		 loop (Some Long) sp
	     | Sshort, Some Short -> 
		 warning "duplicate short"; loop lo sp
	     | Sshort, Some (Long | LongLong) | Slong, Some Short -> 
		 error "both long and short specified"
	     | Slong, Some Long -> 
		 loop (Some LongLong) sp
	     | Slong, Some LongLong ->
		 error "too long for caduceus"
	     | _ -> 
		 assert false)
      | _ :: sp -> 
	  loop lo sp
    in
    loop None

  let rec interp_type specs decl = 
    let st = storage_class specs in
    let sg = sign specs in
    let lg = length specs in
    let rec base_type tyo = function
      | [] -> 
	  (match tyo with 
	     | Some ty -> ty 
	     | None when st = No_storage && sg = None && lg = None -> 
		 error "data definition has no type or storage class"
	     | None -> CTint)
      | Stype t :: sp when tyo = None ->
	  base_type (Some t) sp
      | Sstruct_decl (so, pl) :: sp when tyo = None ->
	  base_type (Some (CTstruct_decl (so, params pl))) sp
      | Sunion_decl (so, pl) :: sp when tyo = None ->
	  base_type (Some (CTunion_decl (so, params pl))) sp
      | (Stype _ | Sstruct_decl _ | Sunion_decl _) :: _ ->
	  error "two or more data types in declaration"
      | _ :: sp ->
	  base_type tyo sp
    and full_type ty = function
      | Dsimple -> ty
      | Dpointer d -> full_type (CTpointer ty) d
      | Darray (d, so) -> CTarray (full_type ty d, so)
      | Dfunction (d, pl, _) -> CTfun (params pl, full_type ty d)
      | Dbitfield (d, e) -> assert false
    and params pl = 
      List.map (fun (s,d,x) -> (interp_type s d, x)) pl
    in
    let ty = full_type (base_type None specs) decl in
    { ctype_expr = ty;
      ctype_storage = st;
      ctype_const = List.exists ((=) Sconst) specs;
      ctype_volatile = List.exists ((=) Svolatile) specs;
      ctype_signed = apply_sign sg ty }

  let interp_param (s, d, id) = interp_type s d, id
  let interp_params = List.map interp_param

  let is_typedef = List.exists ((=) Stypedef)

  let declaration specs decls =
    let l = loc() in
    if is_typedef specs then
      let interp = function
	| (n,d), Inothing -> Ctypes.add n; Ctypedef (interp_type specs d, n)
	| (n,_), _ -> error ("typedef " ^ n ^ " is initialized")
      in
      List.map interp decls
    else
      let interp ((n,d),i) =
	Ctypes.remove n; Cdecl (interp_type specs d, n, i)
      in
      List.map interp decls

# 269 "c/cparser.ml"
let yytransl_const = [|
  263 (* SIZEOF *);
  264 (* PTR_OP *);
  265 (* INC_OP *);
  266 (* DEC_OP *);
  267 (* LEFT_OP *);
  268 (* RIGHT_OP *);
  269 (* LE_OP *);
  270 (* GE_OP *);
  271 (* EQ_OP *);
  272 (* NE_OP *);
  273 (* AND_OP *);
  274 (* OR_OP *);
  275 (* MUL_ASSIGN *);
  276 (* DIV_ASSIGN *);
  277 (* MOD_ASSIGN *);
  278 (* ADD_ASSIGN *);
  279 (* SUB_ASSIGN *);
  280 (* LEFT_ASSIGN *);
  281 (* RIGHT_ASSIGN *);
  282 (* AND_ASSIGN *);
  283 (* XOR_ASSIGN *);
  284 (* OR_ASSIGN *);
  285 (* TYPEDEF *);
  286 (* EXTERN *);
  287 (* STATIC *);
  288 (* AUTO *);
  289 (* REGISTER *);
  290 (* CHAR *);
  291 (* SHORT *);
  292 (* INT *);
  293 (* LONG *);
  294 (* SIGNED *);
  295 (* UNSIGNED *);
  296 (* FLOAT *);
  297 (* DOUBLE *);
  298 (* CONST *);
  299 (* VOLATILE *);
  300 (* VOID *);
  301 (* STRUCT *);
  302 (* UNION *);
  303 (* ENUM *);
  304 (* ELLIPSIS *);
  305 (* CASE *);
  306 (* DEFAULT *);
  307 (* IF *);
  308 (* ELSE *);
  309 (* SWITCH *);
  310 (* WHILE *);
  311 (* DO *);
  312 (* FOR *);
  313 (* GOTO *);
  314 (* CONTINUE *);
  315 (* BREAK *);
  316 (* RETURN *);
  317 (* SEMICOLON *);
  318 (* LBRACE *);
  319 (* RBRACE *);
  320 (* COMMA *);
  321 (* COLON *);
  322 (* EQUAL *);
  323 (* LPAR *);
  324 (* RPAR *);
  325 (* LSQUARE *);
  326 (* RSQUARE *);
  327 (* DOT *);
  328 (* AMP *);
  329 (* EXL *);
  330 (* TILDE *);
  331 (* MINUS *);
  332 (* PLUS *);
  333 (* STAR *);
  334 (* SLASH *);
  335 (* PERCENT *);
  336 (* LT *);
  337 (* GT *);
  338 (* HAT *);
  339 (* PIPE *);
  340 (* QUESTION *);
    0 (* EOF *);
    0|]

let yytransl_block = [|
  257 (* ANNOT *);
  258 (* WDECL *);
  259 (* IDENTIFIER *);
  260 (* CONSTANT *);
  261 (* STRING_LITERAL *);
  262 (* TYPE_NAME *);
    0|]

let yylhs = "\255\255\
\001\000\001\000\003\000\003\000\003\000\003\000\005\000\005\000\
\005\000\005\000\005\000\005\000\005\000\005\000\006\000\006\000\
\009\000\009\000\009\000\009\000\009\000\009\000\010\000\010\000\
\010\000\010\000\010\000\010\000\011\000\011\000\013\000\013\000\
\013\000\013\000\014\000\014\000\014\000\015\000\015\000\015\000\
\016\000\016\000\016\000\016\000\016\000\017\000\017\000\017\000\
\018\000\018\000\019\000\019\000\020\000\020\000\021\000\021\000\
\022\000\022\000\023\000\023\000\008\000\008\000\024\000\024\000\
\024\000\024\000\024\000\024\000\024\000\024\000\024\000\024\000\
\024\000\004\000\004\000\025\000\026\000\026\000\026\000\027\000\
\027\000\027\000\027\000\027\000\027\000\031\000\031\000\031\000\
\031\000\031\000\031\000\028\000\028\000\034\000\034\000\029\000\
\029\000\029\000\029\000\029\000\030\000\030\000\033\000\033\000\
\033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
\033\000\007\000\007\000\037\000\037\000\037\000\039\000\039\000\
\040\000\040\000\041\000\042\000\042\000\042\000\042\000\044\000\
\044\000\044\000\044\000\043\000\043\000\045\000\045\000\045\000\
\038\000\038\000\038\000\046\000\046\000\047\000\047\000\032\000\
\032\000\035\000\035\000\049\000\049\000\049\000\049\000\049\000\
\049\000\049\000\051\000\051\000\048\000\048\000\048\000\048\000\
\053\000\053\000\050\000\050\000\054\000\054\000\055\000\055\000\
\055\000\052\000\052\000\012\000\012\000\056\000\056\000\056\000\
\057\000\057\000\057\000\057\000\057\000\057\000\057\000\057\000\
\057\000\036\000\036\000\036\000\058\000\058\000\059\000\059\000\
\059\000\059\000\059\000\059\000\060\000\060\000\060\000\061\000\
\061\000\061\000\061\000\066\000\069\000\068\000\068\000\067\000\
\067\000\062\000\062\000\062\000\063\000\063\000\063\000\064\000\
\064\000\064\000\064\000\065\000\065\000\065\000\065\000\065\000\
\002\000\002\000\070\000\070\000\071\000\072\000\072\000\072\000\
\072\000\000\000"

let yylen = "\002\000\
\002\000\001\000\001\000\001\000\001\000\003\000\001\000\004\000\
\003\000\004\000\003\000\003\000\002\000\002\000\001\000\003\000\
\001\000\002\000\002\000\002\000\002\000\004\000\001\000\001\000\
\001\000\001\000\001\000\001\000\001\000\004\000\001\000\003\000\
\003\000\003\000\001\000\003\000\003\000\001\000\003\000\003\000\
\001\000\003\000\003\000\003\000\003\000\001\000\003\000\003\000\
\001\000\003\000\001\000\003\000\001\000\003\000\001\000\003\000\
\001\000\003\000\001\000\005\000\001\000\003\000\001\000\001\000\
\001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\001\000\001\000\003\000\001\000\002\000\003\000\001\000\001\000\
\002\000\001\000\002\000\001\000\002\000\001\000\002\000\001\000\
\002\000\001\000\002\000\001\000\003\000\001\000\003\000\001\000\
\001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\001\000\001\000\001\000\005\000\004\000\002\000\001\000\001\000\
\001\000\002\000\003\000\002\000\001\000\002\000\001\000\002\000\
\001\000\002\000\001\000\001\000\003\000\001\000\002\000\003\000\
\004\000\005\000\002\000\001\000\003\000\001\000\003\000\001\000\
\001\000\002\000\001\000\001\000\003\000\004\000\003\000\005\000\
\004\000\004\000\001\000\000\000\001\000\002\000\002\000\003\000\
\001\000\002\000\001\000\003\000\001\000\003\000\002\000\002\000\
\001\000\001\000\003\000\001\000\002\000\001\000\001\000\002\000\
\003\000\002\000\003\000\003\000\004\000\002\000\003\000\003\000\
\004\000\001\000\003\000\004\000\001\000\003\000\001\000\001\000\
\001\000\001\000\001\000\001\000\003\000\004\000\003\000\002\000\
\003\000\003\000\004\000\001\000\002\000\001\000\002\000\001\000\
\002\000\001\000\002\000\002\000\005\000\007\000\005\000\006\000\
\008\000\007\000\008\000\003\000\002\000\002\000\002\000\003\000\
\001\000\002\000\001\000\001\000\002\000\003\000\002\000\003\000\
\002\000\002\000"

let yydefred = "\000\000\
\000\000\000\000\079\000\114\000\000\000\096\000\097\000\098\000\
\099\000\100\000\104\000\105\000\106\000\107\000\110\000\111\000\
\108\000\109\000\144\000\145\000\103\000\119\000\120\000\000\000\
\000\000\000\000\002\000\234\000\000\000\148\000\228\000\000\000\
\000\000\000\000\000\000\101\000\000\000\112\000\113\000\000\000\
\000\000\000\000\225\000\227\000\000\000\115\000\000\000\000\000\
\000\000\161\000\159\000\000\000\001\000\226\000\077\000\000\000\
\092\000\000\000\102\000\081\000\000\000\083\000\000\000\000\000\
\085\000\204\000\206\000\000\000\233\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\229\000\000\000\000\000\
\140\000\000\000\149\000\162\000\160\000\078\000\000\000\000\000\
\000\000\087\000\091\000\089\000\000\000\000\000\000\000\004\000\
\005\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\210\000\
\200\000\000\000\023\000\028\000\027\000\026\000\025\000\024\000\
\007\000\000\000\000\000\000\000\074\000\000\000\000\000\031\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\061\000\208\000\191\000\192\000\193\000\194\000\
\195\000\196\000\000\000\000\000\207\000\232\000\000\000\000\000\
\000\000\121\000\000\000\000\000\170\000\000\000\000\000\000\000\
\000\000\000\000\165\000\003\000\151\000\029\000\076\000\000\000\
\155\000\205\000\000\000\137\000\000\000\000\000\093\000\000\000\
\186\000\095\000\211\000\000\000\021\000\000\000\018\000\019\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\221\000\222\000\223\000\000\000\000\000\000\000\000\000\212\000\
\000\000\000\000\013\000\014\000\000\000\000\000\000\000\000\000\
\064\000\065\000\066\000\067\000\068\000\069\000\070\000\071\000\
\072\000\073\000\063\000\000\000\020\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\201\000\209\000\202\000\000\000\000\000\000\000\124\000\126\000\
\117\000\122\000\000\000\000\000\000\000\132\000\000\000\154\000\
\000\000\000\000\167\000\000\000\168\000\000\000\000\000\000\000\
\153\000\000\000\150\000\143\000\141\000\138\000\189\000\000\000\
\000\000\000\000\199\000\000\000\000\000\000\000\000\000\000\000\
\220\000\224\000\006\000\000\000\000\000\000\000\173\000\075\000\
\012\000\009\000\000\000\015\000\000\000\011\000\197\000\062\000\
\032\000\033\000\034\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\203\000\130\000\128\000\135\000\000\000\
\123\000\000\000\116\000\182\000\000\000\000\000\178\000\000\000\
\000\000\000\000\000\000\152\000\171\000\164\000\166\000\187\000\
\000\000\022\000\198\000\000\000\000\000\000\000\000\000\000\000\
\030\000\000\000\010\000\008\000\000\000\136\000\133\000\183\000\
\177\000\179\000\184\000\000\000\180\000\000\000\188\000\190\000\
\000\000\215\000\000\000\000\000\000\000\000\000\016\000\060\000\
\185\000\181\000\000\000\216\000\000\000\000\000\000\000\214\000\
\000\000\218\000\000\000\217\000\219\000"

let yydgoto = "\002\000\
\028\000\029\000\121\000\122\000\123\000\035\001\124\000\125\000\
\126\000\127\000\128\000\198\000\129\000\130\000\131\000\132\000\
\133\000\134\000\135\000\136\000\137\000\138\000\139\000\220\000\
\168\000\067\000\068\000\056\000\033\000\034\000\062\000\035\000\
\036\000\057\000\037\000\178\000\038\000\039\000\040\000\153\000\
\154\000\155\000\253\000\247\000\254\000\080\000\081\000\041\000\
\042\000\069\001\170\000\161\000\052\000\162\000\163\000\070\001\
\006\001\016\001\140\000\141\000\142\000\143\000\144\000\145\000\
\146\000\070\000\147\000\071\000\078\000\043\000\044\000\045\000"

let yysindex = "\022\000\
\001\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\043\255\
\104\255\040\255\000\000\000\000\020\000\000\000\000\000\116\000\
\240\008\218\008\240\008\000\000\090\008\000\000\000\000\060\255\
\074\255\173\255\000\000\000\000\201\254\000\000\034\255\254\254\
\004\255\000\000\000\000\040\255\000\000\000\000\000\000\045\255\
\000\000\028\008\000\000\000\000\218\008\000\000\240\008\218\008\
\000\000\000\000\000\000\116\000\000\000\225\000\090\008\219\004\
\016\255\173\255\008\008\145\003\117\255\000\000\125\255\084\000\
\000\000\034\255\000\000\000\000\000\000\000\000\104\255\169\003\
\178\008\000\000\000\000\000\000\148\255\083\255\000\000\000\000\
\000\000\039\004\064\004\064\004\091\004\110\255\139\255\154\255\
\168\255\102\002\181\255\098\255\158\255\197\255\186\003\000\000\
\000\000\209\002\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\166\255\171\000\167\255\000\000\116\002\091\004\000\000\
\093\000\098\000\070\000\008\255\160\000\195\255\191\255\242\255\
\108\000\000\255\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\175\001\069\001\000\000\000\000\083\004\219\004\
\185\005\000\000\018\000\219\004\000\000\117\255\070\255\052\000\
\022\255\077\000\000\000\000\000\000\000\000\000\000\000\089\000\
\000\000\000\000\091\004\000\000\034\255\141\000\000\000\169\003\
\000\000\000\000\000\000\209\002\000\000\091\004\000\000\000\000\
\068\000\102\002\091\004\091\004\091\004\162\000\125\000\121\000\
\000\000\000\000\000\000\057\000\072\255\117\000\237\254\000\000\
\091\004\098\255\000\000\000\000\197\003\091\004\098\255\102\002\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\091\004\000\000\091\004\091\004\091\004\
\091\004\091\004\091\004\091\004\091\004\091\004\091\004\091\004\
\091\004\091\004\091\004\091\004\091\004\091\004\091\004\091\004\
\000\000\000\000\000\000\252\001\083\004\083\004\000\000\000\000\
\000\000\000\000\091\004\123\000\063\000\000\000\132\008\000\000\
\100\003\220\003\000\000\061\255\000\000\214\255\117\255\192\000\
\000\000\197\008\000\000\000\000\000\000\000\000\000\000\155\000\
\145\000\102\002\000\000\119\255\135\255\147\255\170\000\125\000\
\000\000\000\000\000\000\091\004\252\005\027\000\000\000\000\000\
\000\000\000\000\160\255\000\000\152\255\000\000\000\000\000\000\
\000\000\000\000\000\000\093\000\093\000\098\000\098\000\070\000\
\070\000\070\000\070\000\008\255\008\255\160\000\195\255\191\255\
\242\255\108\000\156\000\000\000\000\000\000\000\000\000\091\004\
\000\000\018\000\000\000\000\000\165\000\173\000\000\000\169\000\
\214\255\071\008\245\003\000\000\000\000\000\000\000\000\000\000\
\057\002\000\000\000\000\102\002\102\002\242\000\180\000\016\004\
\000\000\091\004\000\000\000\000\091\004\000\000\000\000\000\000\
\000\000\000\000\000\000\184\000\000\000\203\000\000\000\000\000\
\239\000\000\000\102\002\091\004\038\001\165\255\000\000\000\000\
\000\000\000\000\102\002\000\000\186\255\102\002\039\001\000\000\
\243\000\000\000\102\002\000\000\000\000"

let yyrindex = "\000\000\
\000\000\000\000\000\000\000\000\164\002\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\121\255\000\000\000\000\000\000\000\000\000\000\000\000\
\148\000\065\255\248\000\000\000\000\000\000\000\000\000\000\000\
\000\000\173\007\000\000\000\000\000\000\000\000\000\000\002\003\
\000\000\000\000\000\000\139\000\000\000\000\000\000\000\000\000\
\000\000\028\000\000\000\000\000\140\255\000\000\130\001\170\255\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\051\003\219\007\000\000\000\000\069\000\000\000\159\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\241\000\000\000\000\000\000\000\096\000\000\000\139\001\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\158\004\000\000\000\000\131\004\000\000\000\000\
\232\004\098\005\031\006\179\006\036\005\068\007\100\007\126\000\
\214\004\132\255\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\176\255\199\001\
\000\000\000\000\000\000\000\000\000\000\037\007\187\255\000\000\
\000\000\237\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\238\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\085\000\206\001\000\000\000\000\
\000\000\000\000\000\000\103\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\208\255\000\000\006\000\037\007\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\240\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\005\074\005\172\005\196\005\061\006\
\091\006\121\006\151\006\209\006\237\006\134\005\076\007\108\007\
\096\003\061\005\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\016\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\025\002\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000"

let yygindex = "\000\000\
\000\000\000\000\000\000\161\255\000\000\000\000\011\000\171\255\
\182\255\000\000\146\255\127\001\011\001\018\001\164\000\056\001\
\078\001\082\001\077\001\081\001\083\001\000\000\091\000\000\000\
\210\255\032\000\255\255\000\000\251\255\199\255\104\001\097\001\
\236\255\233\001\013\000\088\255\000\000\000\000\000\000\165\001\
\109\255\151\255\000\000\048\001\001\001\242\001\163\001\235\255\
\219\255\192\255\108\255\000\000\000\000\000\000\071\001\127\255\
\035\255\000\000\226\000\000\000\149\000\078\255\000\000\000\000\
\000\000\000\000\190\001\240\255\000\000\054\002\000\000\000\000"

let yytablesize = 2591
let yytable = "\032\000\
\027\000\166\000\177\000\074\000\051\000\250\000\066\000\015\001\
\199\000\000\001\160\000\030\000\024\001\064\000\151\000\196\000\
\221\000\239\000\197\000\053\000\229\000\230\000\001\000\181\000\
\183\000\184\000\166\000\032\000\061\000\005\001\085\000\060\000\
\031\000\065\000\048\000\030\000\079\000\049\000\073\001\030\000\
\064\000\089\000\030\000\064\000\058\000\004\000\248\000\029\001\
\046\000\002\001\073\000\030\000\166\000\148\000\185\000\061\000\
\151\000\026\000\061\000\082\000\031\000\091\000\004\000\004\000\
\073\001\046\000\046\000\082\000\156\000\031\001\082\000\083\000\
\004\000\159\000\199\000\046\000\004\000\156\000\030\000\046\000\
\093\000\019\000\020\000\240\000\197\000\008\001\197\000\231\000\
\232\000\009\001\177\000\020\001\021\001\022\001\151\000\151\000\
\166\000\030\000\151\000\093\000\004\000\088\001\149\000\046\000\
\047\000\086\000\004\000\250\000\087\000\046\000\037\001\041\001\
\042\001\043\001\076\001\032\001\026\000\169\000\192\000\036\001\
\149\000\072\000\151\000\157\000\012\001\082\000\157\000\001\001\
\082\000\002\001\246\000\082\000\082\000\082\000\040\001\201\000\
\001\001\004\001\002\001\027\001\025\000\082\000\086\000\179\000\
\059\001\086\000\026\000\166\000\166\000\166\000\166\000\166\000\
\166\000\166\000\166\000\166\000\166\000\166\000\166\000\166\000\
\166\000\166\000\166\000\166\000\166\000\030\000\167\000\252\000\
\104\001\030\000\025\000\003\001\088\000\089\001\186\000\088\000\
\166\000\030\001\125\000\149\000\026\000\125\000\201\000\166\000\
\157\000\069\000\084\001\157\000\157\000\157\000\171\000\167\000\
\059\000\077\000\059\000\059\000\059\000\151\000\201\000\059\000\
\086\000\059\000\085\001\086\000\063\001\187\000\086\000\086\000\
\086\000\166\000\201\000\072\001\033\001\088\000\086\001\201\000\
\086\000\038\001\193\000\150\000\188\000\092\001\074\000\090\001\
\246\000\246\000\200\000\091\001\201\000\201\000\088\000\208\000\
\119\001\088\000\189\000\004\001\088\000\088\000\088\000\075\000\
\125\000\076\000\125\000\125\000\125\000\166\000\088\000\191\000\
\110\001\201\000\169\000\177\000\125\000\121\001\169\000\159\000\
\166\000\194\000\003\000\004\000\111\001\167\000\005\000\030\001\
\159\000\100\001\235\000\030\000\117\001\049\000\030\000\174\000\
\236\000\094\001\166\000\174\000\004\000\003\000\004\000\046\000\
\074\001\005\000\075\001\159\000\102\001\006\000\007\000\008\000\
\009\000\010\000\011\000\012\000\013\000\014\000\015\000\016\000\
\017\000\018\000\019\000\020\000\021\000\022\000\023\000\024\000\
\006\000\007\000\008\000\009\000\010\000\011\000\012\000\013\000\
\014\000\015\000\016\000\017\000\018\000\019\000\020\000\021\000\
\022\000\023\000\024\000\025\000\237\000\175\000\156\000\156\000\
\159\000\175\000\156\000\190\000\030\000\026\000\252\000\176\000\
\227\000\228\000\251\000\176\000\025\000\167\000\025\000\131\000\
\094\000\231\000\131\000\094\000\167\000\029\001\026\000\002\001\
\026\000\156\000\156\000\156\000\156\000\156\000\156\000\156\000\
\156\000\156\000\156\000\156\000\156\000\156\000\156\000\156\000\
\156\000\156\000\156\000\156\000\242\000\026\001\004\000\007\001\
\201\000\046\000\050\000\065\001\238\000\094\000\066\001\164\000\
\096\000\097\000\063\000\098\000\018\001\099\000\100\000\156\000\
\048\001\049\001\050\001\051\001\010\001\158\000\055\000\055\000\
\158\000\156\000\172\000\173\000\084\000\131\000\080\000\131\000\
\131\000\131\000\167\000\019\001\094\000\063\000\011\001\094\000\
\063\000\131\000\023\001\134\000\090\000\167\000\134\000\092\000\
\152\000\222\000\223\000\224\000\225\000\226\000\233\000\234\000\
\055\000\039\001\202\000\203\000\204\000\025\001\025\000\112\001\
\028\001\112\000\055\000\064\001\055\000\055\000\055\000\114\000\
\026\000\055\000\077\001\055\000\115\000\116\000\117\000\118\000\
\119\000\120\000\158\000\014\001\173\000\158\000\158\000\158\000\
\080\000\055\000\152\000\080\000\082\001\242\000\080\000\080\000\
\080\000\080\001\081\001\201\000\093\001\142\000\142\000\087\001\
\080\000\094\000\003\000\095\000\096\000\097\000\005\000\098\000\
\096\001\099\000\100\000\044\001\045\001\205\000\098\001\206\000\
\097\001\207\000\107\001\083\001\046\001\047\001\108\001\245\000\
\152\000\152\000\084\000\113\001\152\000\006\000\007\000\008\000\
\009\000\010\000\011\000\012\000\013\000\014\000\015\000\016\000\
\017\000\018\000\019\000\020\000\021\000\022\000\023\000\024\000\
\114\001\101\000\102\000\103\000\152\000\104\000\105\000\106\000\
\107\000\108\000\109\000\110\000\111\000\112\000\066\000\113\000\
\052\001\053\001\115\001\114\000\061\001\062\001\118\001\123\001\
\115\000\116\000\117\000\118\000\119\000\120\000\230\000\124\001\
\163\000\172\000\017\001\174\000\084\000\105\001\106\001\084\000\
\054\001\056\001\084\000\084\000\084\000\055\001\057\001\175\000\
\255\000\058\001\095\001\174\000\084\000\094\000\003\000\095\000\
\096\000\097\000\005\000\098\000\116\001\099\000\100\000\013\001\
\079\001\244\000\054\000\000\000\120\001\245\000\245\000\122\001\
\000\000\000\000\000\000\000\000\125\001\000\000\000\000\152\000\
\000\000\006\000\007\000\008\000\009\000\010\000\011\000\012\000\
\013\000\014\000\015\000\016\000\017\000\018\000\019\000\020\000\
\021\000\022\000\023\000\024\000\000\000\101\000\102\000\103\000\
\000\000\104\000\105\000\106\000\107\000\108\000\109\000\110\000\
\111\000\112\000\066\000\243\000\090\000\000\000\000\000\114\000\
\000\000\000\000\000\000\000\000\115\000\116\000\117\000\118\000\
\119\000\120\000\003\000\003\000\003\000\003\000\003\000\003\000\
\003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
\003\000\003\000\003\000\003\000\003\000\003\000\003\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\094\000\
\000\000\095\000\096\000\097\000\046\000\098\000\000\000\099\000\
\100\000\000\000\000\000\000\000\000\000\000\000\090\000\000\000\
\000\000\090\000\000\000\000\000\090\000\090\000\090\000\003\000\
\000\000\127\000\003\000\114\000\003\000\003\000\090\000\003\000\
\129\000\003\000\003\000\129\000\000\000\003\000\003\000\003\000\
\003\000\003\000\003\000\003\000\003\000\003\000\003\000\101\000\
\102\000\103\000\000\000\104\000\105\000\106\000\107\000\108\000\
\109\000\110\000\111\000\112\000\066\000\241\000\000\000\000\000\
\000\000\114\000\000\000\000\000\000\000\000\000\115\000\116\000\
\117\000\118\000\119\000\120\000\094\000\000\000\095\000\096\000\
\097\000\046\000\098\000\000\000\099\000\100\000\000\000\127\000\
\000\000\127\000\127\000\127\000\000\000\000\000\129\000\000\000\
\129\000\129\000\129\000\127\000\000\000\000\000\000\000\000\000\
\000\000\213\000\129\000\213\000\213\000\213\000\213\000\213\000\
\000\000\213\000\213\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\101\000\102\000\103\000\000\000\
\104\000\105\000\106\000\107\000\108\000\109\000\110\000\111\000\
\112\000\066\000\060\001\164\000\096\000\097\000\114\000\098\000\
\000\000\099\000\100\000\115\000\116\000\117\000\118\000\119\000\
\120\000\213\000\213\000\213\000\000\000\213\000\213\000\213\000\
\213\000\213\000\213\000\213\000\213\000\213\000\213\000\213\000\
\000\000\000\000\000\000\213\000\000\000\000\000\000\000\000\000\
\213\000\213\000\213\000\213\000\213\000\213\000\094\000\000\000\
\095\000\096\000\097\000\046\000\098\000\000\000\099\000\100\000\
\000\000\000\000\000\000\000\000\000\000\000\000\176\000\103\001\
\000\000\000\000\000\000\114\000\000\000\000\000\000\000\000\000\
\115\000\116\000\117\000\118\000\119\000\120\000\209\000\210\000\
\211\000\212\000\213\000\214\000\215\000\216\000\217\000\218\000\
\000\000\000\000\000\000\000\000\000\000\000\000\101\000\102\000\
\103\000\000\000\104\000\105\000\106\000\107\000\108\000\109\000\
\110\000\111\000\112\000\066\000\000\000\115\000\102\000\000\000\
\114\000\102\000\000\000\000\000\000\000\115\000\116\000\117\000\
\118\000\119\000\120\000\000\000\000\000\219\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\102\000\102\000\102\000\102\000\102\000\102\000\102\000\102\000\
\102\000\102\000\102\000\102\000\102\000\102\000\102\000\102\000\
\102\000\102\000\102\000\164\000\096\000\097\000\059\000\098\000\
\000\000\099\000\100\000\000\000\000\000\000\000\000\000\000\000\
\102\000\115\000\000\000\102\000\115\000\000\000\102\000\102\000\
\102\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\102\000\000\000\011\000\012\000\013\000\014\000\015\000\016\000\
\017\000\018\000\019\000\020\000\021\000\022\000\023\000\024\000\
\000\000\000\000\000\000\000\000\139\000\000\000\000\000\139\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\114\000\000\000\000\000\000\000\000\000\
\115\000\116\000\117\000\118\000\119\000\120\000\139\000\139\000\
\139\000\139\000\139\000\139\000\139\000\139\000\139\000\139\000\
\139\000\139\000\139\000\139\000\139\000\139\000\139\000\139\000\
\139\000\000\000\000\000\000\000\000\000\118\000\000\000\000\000\
\118\000\000\000\000\000\000\000\000\000\000\000\139\000\000\000\
\000\000\139\000\139\000\000\000\139\000\139\000\139\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\139\000\118\000\
\118\000\118\000\118\000\118\000\118\000\118\000\118\000\118\000\
\118\000\118\000\118\000\118\000\118\000\118\000\118\000\118\000\
\118\000\118\000\000\000\000\000\000\000\000\000\004\000\000\000\
\000\000\005\000\000\000\000\000\000\000\000\000\000\000\118\000\
\056\000\056\000\118\000\118\000\000\000\118\000\118\000\118\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\118\000\
\006\000\007\000\008\000\009\000\010\000\011\000\012\000\013\000\
\014\000\015\000\016\000\017\000\018\000\019\000\020\000\021\000\
\022\000\023\000\024\000\164\000\096\000\097\000\000\000\098\000\
\000\000\099\000\100\000\000\000\056\000\000\000\056\000\056\000\
\056\000\000\000\000\000\056\000\000\000\056\000\001\001\068\001\
\002\001\000\000\000\000\164\000\096\000\097\000\000\000\098\000\
\026\000\099\000\100\000\056\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\164\000\096\000\097\000\000\000\
\098\000\000\000\099\000\100\000\000\000\000\000\000\000\164\000\
\096\000\097\000\000\000\098\000\000\000\099\000\100\000\000\000\
\000\000\000\000\000\000\114\000\000\000\000\000\165\000\000\000\
\115\000\116\000\117\000\118\000\119\000\120\000\164\000\096\000\
\097\000\000\000\098\000\000\000\099\000\100\000\176\000\000\000\
\000\000\000\000\000\000\114\000\000\000\000\000\000\000\000\000\
\115\000\116\000\117\000\118\000\119\000\120\000\195\000\164\000\
\096\000\097\000\000\000\098\000\114\000\099\000\100\000\000\000\
\000\000\115\000\116\000\117\000\118\000\119\000\120\000\114\000\
\034\001\000\000\000\000\000\000\115\000\116\000\117\000\118\000\
\119\000\120\000\164\000\096\000\097\000\000\000\098\000\000\000\
\099\000\100\000\000\000\000\000\000\000\000\000\114\000\000\000\
\000\000\071\001\000\000\115\000\116\000\117\000\118\000\119\000\
\120\000\164\000\096\000\097\000\000\000\098\000\000\000\099\000\
\100\000\000\000\000\000\000\000\000\000\000\000\000\000\114\000\
\000\000\000\000\101\001\000\000\115\000\116\000\117\000\118\000\
\119\000\120\000\164\000\096\000\097\000\000\000\098\000\000\000\
\099\000\100\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\114\000\109\001\000\000\000\000\000\000\115\000\
\116\000\117\000\118\000\119\000\120\000\164\000\096\000\097\000\
\000\000\098\000\000\000\099\000\100\000\000\000\000\000\000\000\
\000\000\180\000\000\000\000\000\000\000\000\000\115\000\116\000\
\117\000\118\000\119\000\120\000\011\000\012\000\013\000\014\000\
\015\000\016\000\017\000\018\000\019\000\020\000\021\000\022\000\
\023\000\024\000\182\000\000\000\000\000\000\000\000\000\115\000\
\116\000\117\000\118\000\119\000\120\000\029\000\029\000\029\000\
\029\000\029\000\029\000\029\000\029\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\114\000\000\000\000\000\
\000\000\000\000\115\000\116\000\117\000\118\000\119\000\120\000\
\017\000\017\000\017\000\017\000\017\000\017\000\017\000\017\000\
\017\000\017\000\017\000\017\000\017\000\017\000\017\000\017\000\
\017\000\017\000\000\000\000\000\000\000\000\000\000\000\029\000\
\000\000\029\000\029\000\029\000\000\000\000\000\029\000\000\000\
\029\000\000\000\029\000\000\000\000\000\029\000\029\000\029\000\
\029\000\029\000\029\000\029\000\029\000\029\000\029\000\000\000\
\000\000\000\000\017\000\000\000\017\000\017\000\017\000\017\000\
\059\000\017\000\000\000\017\000\000\000\017\000\000\000\057\000\
\017\000\017\000\017\000\017\000\017\000\017\000\017\000\017\000\
\017\000\017\000\035\000\035\000\035\000\035\000\035\000\035\000\
\035\000\035\000\000\000\000\000\011\000\012\000\013\000\014\000\
\015\000\016\000\017\000\018\000\019\000\020\000\021\000\022\000\
\023\000\024\000\037\000\037\000\037\000\037\000\037\000\037\000\
\037\000\037\000\057\000\000\000\057\000\057\000\057\000\000\000\
\000\000\057\000\000\000\057\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\035\000\000\000\035\000\035\000\
\035\000\057\000\000\000\035\000\000\000\035\000\000\000\035\000\
\000\000\000\000\035\000\035\000\049\000\049\000\000\000\035\000\
\035\000\035\000\035\000\035\000\037\000\000\000\037\000\037\000\
\037\000\000\000\000\000\037\000\000\000\037\000\000\000\037\000\
\000\000\000\000\037\000\037\000\000\000\000\000\058\000\037\000\
\037\000\037\000\037\000\037\000\036\000\036\000\036\000\036\000\
\036\000\036\000\036\000\036\000\000\000\000\000\000\000\000\000\
\049\000\000\000\049\000\049\000\049\000\000\000\000\000\049\000\
\000\000\049\000\000\000\049\000\038\000\038\000\038\000\038\000\
\038\000\038\000\038\000\038\000\000\000\049\000\049\000\049\000\
\000\000\058\000\000\000\058\000\058\000\058\000\000\000\000\000\
\058\000\000\000\058\000\000\000\000\000\000\000\036\000\000\000\
\036\000\036\000\036\000\000\000\000\000\036\000\000\000\036\000\
\058\000\036\000\000\000\000\000\036\000\036\000\050\000\050\000\
\000\000\036\000\036\000\036\000\036\000\036\000\038\000\000\000\
\038\000\038\000\038\000\000\000\000\000\038\000\000\000\038\000\
\000\000\038\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\038\000\038\000\038\000\038\000\038\000\039\000\039\000\
\039\000\039\000\039\000\039\000\039\000\039\000\059\000\000\000\
\000\000\000\000\050\000\000\000\050\000\050\000\050\000\000\000\
\000\000\050\000\000\000\050\000\000\000\050\000\040\000\040\000\
\040\000\040\000\040\000\040\000\040\000\040\000\000\000\050\000\
\050\000\050\000\011\000\012\000\013\000\014\000\015\000\016\000\
\017\000\018\000\019\000\020\000\021\000\022\000\023\000\024\000\
\039\000\000\000\039\000\039\000\039\000\000\000\000\000\039\000\
\000\000\039\000\000\000\039\000\000\000\000\000\000\000\249\000\
\000\000\000\000\000\000\039\000\039\000\039\000\039\000\039\000\
\040\000\059\000\040\000\040\000\040\000\000\000\000\000\040\000\
\000\000\040\000\000\000\040\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\040\000\040\000\040\000\040\000\040\000\
\006\000\007\000\008\000\009\000\010\000\011\000\012\000\013\000\
\014\000\015\000\016\000\017\000\018\000\019\000\020\000\021\000\
\022\000\023\000\024\000\041\000\041\000\041\000\041\000\041\000\
\041\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\029\001\068\001\
\002\001\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\026\000\044\000\044\000\044\000\044\000\044\000\044\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\041\000\000\000\041\000\041\000\041\000\
\000\000\000\000\041\000\000\000\041\000\000\000\041\000\045\000\
\045\000\045\000\045\000\045\000\045\000\000\000\041\000\041\000\
\041\000\041\000\041\000\000\000\000\000\000\000\000\000\000\000\
\000\000\044\000\000\000\044\000\044\000\044\000\000\000\000\000\
\044\000\000\000\044\000\000\000\044\000\042\000\042\000\042\000\
\042\000\042\000\042\000\000\000\044\000\044\000\044\000\044\000\
\044\000\000\000\000\000\000\000\000\000\000\000\000\000\045\000\
\000\000\045\000\045\000\045\000\000\000\000\000\045\000\000\000\
\045\000\000\000\045\000\043\000\043\000\043\000\043\000\043\000\
\043\000\000\000\045\000\045\000\045\000\045\000\045\000\000\000\
\000\000\000\000\000\000\000\000\000\000\042\000\000\000\042\000\
\042\000\042\000\000\000\000\000\042\000\000\000\042\000\000\000\
\042\000\046\000\046\000\046\000\046\000\000\000\000\000\000\000\
\042\000\042\000\042\000\042\000\042\000\000\000\000\000\000\000\
\000\000\000\000\000\000\043\000\000\000\043\000\043\000\043\000\
\000\000\000\000\043\000\000\000\043\000\000\000\043\000\047\000\
\047\000\047\000\047\000\000\000\000\000\000\000\043\000\043\000\
\043\000\043\000\043\000\000\000\000\000\000\000\000\000\046\000\
\000\000\046\000\046\000\046\000\000\000\000\000\046\000\000\000\
\046\000\000\000\046\000\048\000\048\000\048\000\048\000\000\000\
\000\000\000\000\000\000\000\000\046\000\046\000\046\000\000\000\
\000\000\000\000\000\000\000\000\000\000\047\000\000\000\047\000\
\047\000\047\000\000\000\000\000\047\000\000\000\047\000\000\000\
\047\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\047\000\047\000\047\000\000\000\156\000\000\000\
\000\000\048\000\156\000\048\000\048\000\048\000\000\000\000\000\
\048\000\000\000\048\000\000\000\048\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\048\000\048\000\
\048\000\156\000\156\000\156\000\156\000\156\000\156\000\156\000\
\156\000\156\000\156\000\156\000\156\000\156\000\156\000\156\000\
\156\000\156\000\156\000\156\000\051\000\051\000\000\000\000\000\
\000\000\000\000\000\000\000\000\052\000\052\000\000\000\000\000\
\000\000\156\000\156\000\000\000\156\000\156\000\156\000\156\000\
\156\000\156\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\053\000\053\000\000\000\000\000\
\000\000\000\000\000\000\000\000\054\000\054\000\000\000\000\000\
\051\000\000\000\051\000\051\000\051\000\000\000\000\000\051\000\
\052\000\051\000\052\000\052\000\052\000\000\000\000\000\052\000\
\000\000\052\000\000\000\000\000\000\000\051\000\051\000\051\000\
\000\000\000\000\000\000\000\000\000\000\052\000\052\000\052\000\
\053\000\000\000\053\000\053\000\053\000\000\000\000\000\053\000\
\054\000\053\000\054\000\054\000\054\000\000\000\147\000\054\000\
\000\000\054\000\147\000\000\000\000\000\000\000\053\000\053\000\
\000\000\000\000\000\000\000\000\000\000\000\000\054\000\054\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\147\000\147\000\147\000\147\000\147\000\147\000\147\000\
\147\000\147\000\147\000\147\000\147\000\147\000\147\000\147\000\
\147\000\147\000\147\000\147\000\146\000\000\000\000\000\000\000\
\146\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\147\000\147\000\000\000\147\000\147\000\147\000\000\000\
\147\000\000\000\000\000\000\000\000\000\000\000\000\000\146\000\
\146\000\146\000\146\000\146\000\146\000\146\000\146\000\146\000\
\146\000\146\000\146\000\146\000\146\000\146\000\146\000\146\000\
\146\000\146\000\157\000\000\000\000\000\059\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\146\000\
\146\000\000\000\146\000\146\000\146\000\003\000\146\000\000\000\
\000\000\059\000\000\000\000\000\006\000\007\000\008\000\009\000\
\010\000\011\000\012\000\013\000\014\000\015\000\016\000\017\000\
\018\000\019\000\020\000\021\000\022\000\023\000\024\000\000\000\
\006\000\007\000\008\000\009\000\010\000\011\000\012\000\013\000\
\014\000\015\000\016\000\017\000\018\000\019\000\020\000\021\000\
\022\000\023\000\024\000\158\000\059\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\003\000\000\000\088\000\000\000\059\000\
\000\000\000\000\000\000\006\000\007\000\008\000\009\000\010\000\
\011\000\012\000\013\000\014\000\015\000\016\000\017\000\018\000\
\019\000\020\000\021\000\022\000\023\000\024\000\006\000\007\000\
\008\000\009\000\010\000\011\000\012\000\013\000\014\000\015\000\
\016\000\017\000\018\000\019\000\020\000\021\000\022\000\023\000\
\024\000\059\000\099\001\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\066\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\011\000\012\000\013\000\
\014\000\015\000\016\000\017\000\018\000\019\000\020\000\021\000\
\022\000\023\000\024\000\003\000\000\000\000\000\000\000\059\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\067\001\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\059\000\000\000\000\000\000\000\006\000\007\000\
\008\000\009\000\010\000\011\000\012\000\013\000\014\000\015\000\
\016\000\017\000\018\000\019\000\020\000\021\000\022\000\023\000\
\024\000\006\000\007\000\008\000\009\000\010\000\011\000\012\000\
\013\000\014\000\015\000\016\000\017\000\018\000\019\000\020\000\
\021\000\022\000\023\000\024\000\078\001\059\000\006\000\007\000\
\008\000\009\000\010\000\011\000\012\000\013\000\014\000\015\000\
\016\000\017\000\018\000\019\000\020\000\021\000\022\000\023\000\
\024\000\000\000\000\000\000\000\006\000\007\000\008\000\009\000\
\010\000\011\000\012\000\013\000\014\000\015\000\016\000\017\000\
\018\000\019\000\020\000\021\000\022\000\023\000\024\000"

let yycheck = "\001\000\
\000\000\076\000\088\000\041\000\026\000\153\000\062\001\176\000\
\114\000\158\000\075\000\001\000\191\000\034\000\072\000\111\000\
\127\000\018\001\114\000\000\000\013\001\014\001\001\000\098\000\
\099\000\100\000\101\000\029\000\034\000\159\000\052\000\033\000\
\001\000\035\000\024\000\025\000\003\001\025\000\004\001\029\000\
\061\000\058\000\032\000\064\000\032\000\003\001\152\000\067\001\
\006\001\069\001\040\000\041\000\127\000\070\000\101\000\061\000\
\114\000\077\001\064\000\062\001\029\000\063\000\003\001\003\001\
\030\001\006\001\006\001\003\001\000\000\199\000\006\001\068\001\
\003\001\075\000\180\000\006\001\003\001\062\001\068\000\006\001\
\068\000\042\001\043\001\084\001\180\000\064\001\182\000\080\001\
\081\001\068\001\176\000\187\000\188\000\189\000\152\000\153\000\
\171\000\087\000\156\000\087\000\003\001\024\001\071\000\006\001\
\062\001\061\001\003\001\255\000\064\001\006\001\206\000\222\000\
\223\000\224\000\007\001\201\000\077\001\001\001\108\000\205\000\
\089\000\062\001\180\000\003\001\171\000\061\001\006\001\067\001\
\064\001\069\001\151\000\067\001\068\001\069\001\220\000\064\001\
\067\001\159\000\069\001\068\001\067\001\077\001\003\001\061\001\
\240\000\006\001\077\001\222\000\223\000\224\000\225\000\226\000\
\227\000\228\000\229\000\230\000\231\000\232\000\233\000\234\000\
\235\000\236\000\237\000\238\000\239\000\155\000\076\000\155\000\
\081\001\159\000\067\001\159\000\003\001\028\001\065\001\006\001\
\251\000\199\000\003\001\148\000\077\001\006\001\064\001\002\001\
\064\001\037\000\068\001\067\001\068\001\069\001\066\001\101\000\
\061\001\045\000\063\001\064\001\065\001\255\000\064\001\068\001\
\061\001\070\001\068\001\064\001\251\000\067\001\067\001\068\001\
\069\001\028\001\064\001\002\001\202\000\066\001\068\001\064\001\
\077\001\207\000\061\001\071\000\067\001\070\001\004\001\064\001\
\245\000\246\000\061\001\068\001\064\001\064\001\061\001\065\001\
\068\001\064\001\067\001\001\001\067\001\068\001\069\001\067\001\
\065\001\069\001\067\001\068\001\069\001\064\001\077\001\067\001\
\088\001\064\001\064\001\081\001\077\001\068\001\068\001\001\001\
\075\001\061\001\002\001\003\001\090\001\171\000\006\001\029\001\
\010\001\074\001\072\001\001\001\108\001\001\001\004\001\064\001\
\082\001\064\001\093\001\068\001\003\001\002\001\003\001\006\001\
\067\001\006\001\069\001\029\001\075\001\029\001\030\001\031\001\
\032\001\033\001\034\001\035\001\036\001\037\001\038\001\039\001\
\040\001\041\001\042\001\043\001\044\001\045\001\046\001\047\001\
\029\001\030\001\031\001\032\001\033\001\034\001\035\001\036\001\
\037\001\038\001\039\001\040\001\041\001\042\001\043\001\044\001\
\045\001\046\001\047\001\067\001\083\001\064\001\002\001\003\001\
\074\001\068\001\006\001\106\000\066\001\077\001\066\001\064\001\
\011\001\012\001\065\001\068\001\067\001\251\000\067\001\003\001\
\061\001\062\001\006\001\064\001\002\001\067\001\077\001\069\001\
\077\001\029\001\030\001\031\001\032\001\033\001\034\001\035\001\
\036\001\037\001\038\001\039\001\040\001\041\001\042\001\043\001\
\044\001\045\001\046\001\047\001\147\000\061\001\003\001\068\001\
\064\001\006\001\026\000\061\001\017\001\001\001\064\001\003\001\
\004\001\005\001\034\000\007\001\065\001\009\001\010\001\067\001\
\229\000\230\000\231\000\232\000\064\001\003\001\017\001\018\001\
\006\001\077\001\063\001\064\001\052\000\065\001\003\001\067\001\
\068\001\069\001\064\001\186\000\061\001\061\000\070\001\064\001\
\064\000\077\001\001\001\061\001\061\000\075\001\064\001\064\000\
\072\000\077\001\078\001\079\001\075\001\076\001\015\001\016\001\
\061\001\208\000\008\001\009\001\010\001\061\001\067\001\093\001\
\068\001\061\001\061\001\065\001\063\001\064\001\065\001\067\001\
\077\001\068\001\003\001\070\001\072\001\073\001\074\001\075\001\
\076\001\077\001\064\001\063\001\064\001\067\001\068\001\069\001\
\061\001\084\001\114\000\064\001\068\001\244\000\067\001\068\001\
\069\001\063\001\064\001\064\001\065\001\063\001\064\001\054\001\
\077\001\001\001\002\001\003\001\004\001\005\001\006\001\007\001\
\068\001\009\001\010\001\225\000\226\000\067\001\070\001\069\001\
\068\001\071\001\001\001\018\001\227\000\228\000\067\001\151\000\
\152\000\153\000\003\001\068\001\156\000\029\001\030\001\031\001\
\032\001\033\001\034\001\035\001\036\001\037\001\038\001\039\001\
\040\001\041\001\042\001\043\001\044\001\045\001\046\001\047\001\
\070\001\049\001\050\001\051\001\180\000\053\001\054\001\055\001\
\056\001\057\001\058\001\059\001\060\001\061\001\062\001\063\001\
\233\000\234\000\052\001\067\001\245\000\246\000\001\001\001\001\
\072\001\073\001\074\001\075\001\076\001\077\001\062\001\061\001\
\068\001\068\001\180\000\068\001\061\001\084\001\085\001\064\001\
\235\000\237\000\067\001\068\001\069\001\236\000\238\000\087\000\
\156\000\239\000\066\001\082\000\077\001\001\001\002\001\003\001\
\004\001\005\001\006\001\007\001\107\001\009\001\010\001\173\000\
\010\001\148\000\029\000\255\255\115\001\245\000\246\000\118\001\
\255\255\255\255\255\255\255\255\123\001\255\255\255\255\255\000\
\255\255\029\001\030\001\031\001\032\001\033\001\034\001\035\001\
\036\001\037\001\038\001\039\001\040\001\041\001\042\001\043\001\
\044\001\045\001\046\001\047\001\255\255\049\001\050\001\051\001\
\255\255\053\001\054\001\055\001\056\001\057\001\058\001\059\001\
\060\001\061\001\062\001\063\001\003\001\255\255\255\255\067\001\
\255\255\255\255\255\255\255\255\072\001\073\001\074\001\075\001\
\076\001\077\001\008\001\009\001\010\001\011\001\012\001\013\001\
\014\001\015\001\016\001\017\001\018\001\019\001\020\001\021\001\
\022\001\023\001\024\001\025\001\026\001\027\001\028\001\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\001\001\
\255\255\003\001\004\001\005\001\006\001\007\001\255\255\009\001\
\010\001\255\255\255\255\255\255\255\255\255\255\061\001\255\255\
\255\255\064\001\255\255\255\255\067\001\068\001\069\001\061\001\
\255\255\003\001\064\001\065\001\066\001\067\001\077\001\069\001\
\003\001\071\001\072\001\006\001\255\255\075\001\076\001\077\001\
\078\001\079\001\080\001\081\001\082\001\083\001\084\001\049\001\
\050\001\051\001\255\255\053\001\054\001\055\001\056\001\057\001\
\058\001\059\001\060\001\061\001\062\001\063\001\255\255\255\255\
\255\255\067\001\255\255\255\255\255\255\255\255\072\001\073\001\
\074\001\075\001\076\001\077\001\001\001\255\255\003\001\004\001\
\005\001\006\001\007\001\255\255\009\001\010\001\255\255\065\001\
\255\255\067\001\068\001\069\001\255\255\255\255\065\001\255\255\
\067\001\068\001\069\001\077\001\255\255\255\255\255\255\255\255\
\255\255\001\001\077\001\003\001\004\001\005\001\006\001\007\001\
\255\255\009\001\010\001\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\049\001\050\001\051\001\255\255\
\053\001\054\001\055\001\056\001\057\001\058\001\059\001\060\001\
\061\001\062\001\063\001\003\001\004\001\005\001\067\001\007\001\
\255\255\009\001\010\001\072\001\073\001\074\001\075\001\076\001\
\077\001\049\001\050\001\051\001\255\255\053\001\054\001\055\001\
\056\001\057\001\058\001\059\001\060\001\061\001\062\001\063\001\
\255\255\255\255\255\255\067\001\255\255\255\255\255\255\255\255\
\072\001\073\001\074\001\075\001\076\001\077\001\001\001\255\255\
\003\001\004\001\005\001\006\001\007\001\255\255\009\001\010\001\
\255\255\255\255\255\255\255\255\255\255\255\255\062\001\063\001\
\255\255\255\255\255\255\067\001\255\255\255\255\255\255\255\255\
\072\001\073\001\074\001\075\001\076\001\077\001\019\001\020\001\
\021\001\022\001\023\001\024\001\025\001\026\001\027\001\028\001\
\255\255\255\255\255\255\255\255\255\255\255\255\049\001\050\001\
\051\001\255\255\053\001\054\001\055\001\056\001\057\001\058\001\
\059\001\060\001\061\001\062\001\255\255\002\001\003\001\255\255\
\067\001\006\001\255\255\255\255\255\255\072\001\073\001\074\001\
\075\001\076\001\077\001\255\255\255\255\066\001\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\029\001\030\001\031\001\032\001\033\001\034\001\035\001\036\001\
\037\001\038\001\039\001\040\001\041\001\042\001\043\001\044\001\
\045\001\046\001\047\001\003\001\004\001\005\001\006\001\007\001\
\255\255\009\001\010\001\255\255\255\255\255\255\255\255\255\255\
\061\001\062\001\255\255\064\001\065\001\255\255\067\001\068\001\
\069\001\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\077\001\255\255\034\001\035\001\036\001\037\001\038\001\039\001\
\040\001\041\001\042\001\043\001\044\001\045\001\046\001\047\001\
\255\255\255\255\255\255\255\255\003\001\255\255\255\255\006\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\067\001\255\255\255\255\255\255\255\255\
\072\001\073\001\074\001\075\001\076\001\077\001\029\001\030\001\
\031\001\032\001\033\001\034\001\035\001\036\001\037\001\038\001\
\039\001\040\001\041\001\042\001\043\001\044\001\045\001\046\001\
\047\001\255\255\255\255\255\255\255\255\003\001\255\255\255\255\
\006\001\255\255\255\255\255\255\255\255\255\255\061\001\255\255\
\255\255\064\001\065\001\255\255\067\001\068\001\069\001\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\077\001\029\001\
\030\001\031\001\032\001\033\001\034\001\035\001\036\001\037\001\
\038\001\039\001\040\001\041\001\042\001\043\001\044\001\045\001\
\046\001\047\001\255\255\255\255\255\255\255\255\003\001\255\255\
\255\255\006\001\255\255\255\255\255\255\255\255\255\255\061\001\
\017\001\018\001\064\001\065\001\255\255\067\001\068\001\069\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\077\001\
\029\001\030\001\031\001\032\001\033\001\034\001\035\001\036\001\
\037\001\038\001\039\001\040\001\041\001\042\001\043\001\044\001\
\045\001\046\001\047\001\003\001\004\001\005\001\255\255\007\001\
\255\255\009\001\010\001\255\255\061\001\255\255\063\001\064\001\
\065\001\255\255\255\255\068\001\255\255\070\001\067\001\068\001\
\069\001\255\255\255\255\003\001\004\001\005\001\255\255\007\001\
\077\001\009\001\010\001\084\001\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\003\001\004\001\005\001\255\255\
\007\001\255\255\009\001\010\001\255\255\255\255\255\255\003\001\
\004\001\005\001\255\255\007\001\255\255\009\001\010\001\255\255\
\255\255\255\255\255\255\067\001\255\255\255\255\070\001\255\255\
\072\001\073\001\074\001\075\001\076\001\077\001\003\001\004\001\
\005\001\255\255\007\001\255\255\009\001\010\001\062\001\255\255\
\255\255\255\255\255\255\067\001\255\255\255\255\255\255\255\255\
\072\001\073\001\074\001\075\001\076\001\077\001\061\001\003\001\
\004\001\005\001\255\255\007\001\067\001\009\001\010\001\255\255\
\255\255\072\001\073\001\074\001\075\001\076\001\077\001\067\001\
\068\001\255\255\255\255\255\255\072\001\073\001\074\001\075\001\
\076\001\077\001\003\001\004\001\005\001\255\255\007\001\255\255\
\009\001\010\001\255\255\255\255\255\255\255\255\067\001\255\255\
\255\255\070\001\255\255\072\001\073\001\074\001\075\001\076\001\
\077\001\003\001\004\001\005\001\255\255\007\001\255\255\009\001\
\010\001\255\255\255\255\255\255\255\255\255\255\255\255\067\001\
\255\255\255\255\070\001\255\255\072\001\073\001\074\001\075\001\
\076\001\077\001\003\001\004\001\005\001\255\255\007\001\255\255\
\009\001\010\001\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\067\001\068\001\255\255\255\255\255\255\072\001\
\073\001\074\001\075\001\076\001\077\001\003\001\004\001\005\001\
\255\255\007\001\255\255\009\001\010\001\255\255\255\255\255\255\
\255\255\067\001\255\255\255\255\255\255\255\255\072\001\073\001\
\074\001\075\001\076\001\077\001\034\001\035\001\036\001\037\001\
\038\001\039\001\040\001\041\001\042\001\043\001\044\001\045\001\
\046\001\047\001\067\001\255\255\255\255\255\255\255\255\072\001\
\073\001\074\001\075\001\076\001\077\001\011\001\012\001\013\001\
\014\001\015\001\016\001\017\001\018\001\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\067\001\255\255\255\255\
\255\255\255\255\072\001\073\001\074\001\075\001\076\001\077\001\
\011\001\012\001\013\001\014\001\015\001\016\001\017\001\018\001\
\019\001\020\001\021\001\022\001\023\001\024\001\025\001\026\001\
\027\001\028\001\255\255\255\255\255\255\255\255\255\255\061\001\
\255\255\063\001\064\001\065\001\255\255\255\255\068\001\255\255\
\070\001\255\255\072\001\255\255\255\255\075\001\076\001\077\001\
\078\001\079\001\080\001\081\001\082\001\083\001\084\001\255\255\
\255\255\255\255\061\001\255\255\063\001\064\001\065\001\066\001\
\006\001\068\001\255\255\070\001\255\255\072\001\255\255\018\001\
\075\001\076\001\077\001\078\001\079\001\080\001\081\001\082\001\
\083\001\084\001\011\001\012\001\013\001\014\001\015\001\016\001\
\017\001\018\001\255\255\255\255\034\001\035\001\036\001\037\001\
\038\001\039\001\040\001\041\001\042\001\043\001\044\001\045\001\
\046\001\047\001\011\001\012\001\013\001\014\001\015\001\016\001\
\017\001\018\001\061\001\255\255\063\001\064\001\065\001\255\255\
\255\255\068\001\255\255\070\001\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\061\001\255\255\063\001\064\001\
\065\001\084\001\255\255\068\001\255\255\070\001\255\255\072\001\
\255\255\255\255\075\001\076\001\017\001\018\001\255\255\080\001\
\081\001\082\001\083\001\084\001\061\001\255\255\063\001\064\001\
\065\001\255\255\255\255\068\001\255\255\070\001\255\255\072\001\
\255\255\255\255\075\001\076\001\255\255\255\255\018\001\080\001\
\081\001\082\001\083\001\084\001\011\001\012\001\013\001\014\001\
\015\001\016\001\017\001\018\001\255\255\255\255\255\255\255\255\
\061\001\255\255\063\001\064\001\065\001\255\255\255\255\068\001\
\255\255\070\001\255\255\072\001\011\001\012\001\013\001\014\001\
\015\001\016\001\017\001\018\001\255\255\082\001\083\001\084\001\
\255\255\061\001\255\255\063\001\064\001\065\001\255\255\255\255\
\068\001\255\255\070\001\255\255\255\255\255\255\061\001\255\255\
\063\001\064\001\065\001\255\255\255\255\068\001\255\255\070\001\
\084\001\072\001\255\255\255\255\075\001\076\001\017\001\018\001\
\255\255\080\001\081\001\082\001\083\001\084\001\061\001\255\255\
\063\001\064\001\065\001\255\255\255\255\068\001\255\255\070\001\
\255\255\072\001\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\080\001\081\001\082\001\083\001\084\001\011\001\012\001\
\013\001\014\001\015\001\016\001\017\001\018\001\006\001\255\255\
\255\255\255\255\061\001\255\255\063\001\064\001\065\001\255\255\
\255\255\068\001\255\255\070\001\255\255\072\001\011\001\012\001\
\013\001\014\001\015\001\016\001\017\001\018\001\255\255\082\001\
\083\001\084\001\034\001\035\001\036\001\037\001\038\001\039\001\
\040\001\041\001\042\001\043\001\044\001\045\001\046\001\047\001\
\061\001\255\255\063\001\064\001\065\001\255\255\255\255\068\001\
\255\255\070\001\255\255\072\001\255\255\255\255\255\255\063\001\
\255\255\255\255\255\255\080\001\081\001\082\001\083\001\084\001\
\061\001\006\001\063\001\064\001\065\001\255\255\255\255\068\001\
\255\255\070\001\255\255\072\001\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\080\001\081\001\082\001\083\001\084\001\
\029\001\030\001\031\001\032\001\033\001\034\001\035\001\036\001\
\037\001\038\001\039\001\040\001\041\001\042\001\043\001\044\001\
\045\001\046\001\047\001\013\001\014\001\015\001\016\001\017\001\
\018\001\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\067\001\068\001\
\069\001\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\077\001\013\001\014\001\015\001\016\001\017\001\018\001\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\061\001\255\255\063\001\064\001\065\001\
\255\255\255\255\068\001\255\255\070\001\255\255\072\001\013\001\
\014\001\015\001\016\001\017\001\018\001\255\255\080\001\081\001\
\082\001\083\001\084\001\255\255\255\255\255\255\255\255\255\255\
\255\255\061\001\255\255\063\001\064\001\065\001\255\255\255\255\
\068\001\255\255\070\001\255\255\072\001\013\001\014\001\015\001\
\016\001\017\001\018\001\255\255\080\001\081\001\082\001\083\001\
\084\001\255\255\255\255\255\255\255\255\255\255\255\255\061\001\
\255\255\063\001\064\001\065\001\255\255\255\255\068\001\255\255\
\070\001\255\255\072\001\013\001\014\001\015\001\016\001\017\001\
\018\001\255\255\080\001\081\001\082\001\083\001\084\001\255\255\
\255\255\255\255\255\255\255\255\255\255\061\001\255\255\063\001\
\064\001\065\001\255\255\255\255\068\001\255\255\070\001\255\255\
\072\001\015\001\016\001\017\001\018\001\255\255\255\255\255\255\
\080\001\081\001\082\001\083\001\084\001\255\255\255\255\255\255\
\255\255\255\255\255\255\061\001\255\255\063\001\064\001\065\001\
\255\255\255\255\068\001\255\255\070\001\255\255\072\001\015\001\
\016\001\017\001\018\001\255\255\255\255\255\255\080\001\081\001\
\082\001\083\001\084\001\255\255\255\255\255\255\255\255\061\001\
\255\255\063\001\064\001\065\001\255\255\255\255\068\001\255\255\
\070\001\255\255\072\001\015\001\016\001\017\001\018\001\255\255\
\255\255\255\255\255\255\255\255\082\001\083\001\084\001\255\255\
\255\255\255\255\255\255\255\255\255\255\061\001\255\255\063\001\
\064\001\065\001\255\255\255\255\068\001\255\255\070\001\255\255\
\072\001\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\082\001\083\001\084\001\255\255\002\001\255\255\
\255\255\061\001\006\001\063\001\064\001\065\001\255\255\255\255\
\068\001\255\255\070\001\255\255\072\001\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\082\001\083\001\
\084\001\029\001\030\001\031\001\032\001\033\001\034\001\035\001\
\036\001\037\001\038\001\039\001\040\001\041\001\042\001\043\001\
\044\001\045\001\046\001\047\001\017\001\018\001\255\255\255\255\
\255\255\255\255\255\255\255\255\017\001\018\001\255\255\255\255\
\255\255\061\001\062\001\255\255\064\001\065\001\066\001\067\001\
\068\001\069\001\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\017\001\018\001\255\255\255\255\
\255\255\255\255\255\255\255\255\017\001\018\001\255\255\255\255\
\061\001\255\255\063\001\064\001\065\001\255\255\255\255\068\001\
\061\001\070\001\063\001\064\001\065\001\255\255\255\255\068\001\
\255\255\070\001\255\255\255\255\255\255\082\001\083\001\084\001\
\255\255\255\255\255\255\255\255\255\255\082\001\083\001\084\001\
\061\001\255\255\063\001\064\001\065\001\255\255\255\255\068\001\
\061\001\070\001\063\001\064\001\065\001\255\255\002\001\068\001\
\255\255\070\001\006\001\255\255\255\255\255\255\083\001\084\001\
\255\255\255\255\255\255\255\255\255\255\255\255\083\001\084\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\029\001\030\001\031\001\032\001\033\001\034\001\035\001\
\036\001\037\001\038\001\039\001\040\001\041\001\042\001\043\001\
\044\001\045\001\046\001\047\001\002\001\255\255\255\255\255\255\
\006\001\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\061\001\062\001\255\255\064\001\065\001\066\001\255\255\
\068\001\255\255\255\255\255\255\255\255\255\255\255\255\029\001\
\030\001\031\001\032\001\033\001\034\001\035\001\036\001\037\001\
\038\001\039\001\040\001\041\001\042\001\043\001\044\001\045\001\
\046\001\047\001\003\001\255\255\255\255\006\001\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\061\001\
\062\001\255\255\064\001\065\001\066\001\002\001\068\001\255\255\
\255\255\006\001\255\255\255\255\029\001\030\001\031\001\032\001\
\033\001\034\001\035\001\036\001\037\001\038\001\039\001\040\001\
\041\001\042\001\043\001\044\001\045\001\046\001\047\001\255\255\
\029\001\030\001\031\001\032\001\033\001\034\001\035\001\036\001\
\037\001\038\001\039\001\040\001\041\001\042\001\043\001\044\001\
\045\001\046\001\047\001\068\001\006\001\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\002\001\255\255\066\001\255\255\006\001\
\255\255\255\255\255\255\029\001\030\001\031\001\032\001\033\001\
\034\001\035\001\036\001\037\001\038\001\039\001\040\001\041\001\
\042\001\043\001\044\001\045\001\046\001\047\001\029\001\030\001\
\031\001\032\001\033\001\034\001\035\001\036\001\037\001\038\001\
\039\001\040\001\041\001\042\001\043\001\044\001\045\001\046\001\
\047\001\006\001\068\001\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\062\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\034\001\035\001\036\001\
\037\001\038\001\039\001\040\001\041\001\042\001\043\001\044\001\
\045\001\046\001\047\001\002\001\255\255\255\255\255\255\006\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\063\001\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\006\001\255\255\255\255\255\255\029\001\030\001\
\031\001\032\001\033\001\034\001\035\001\036\001\037\001\038\001\
\039\001\040\001\041\001\042\001\043\001\044\001\045\001\046\001\
\047\001\029\001\030\001\031\001\032\001\033\001\034\001\035\001\
\036\001\037\001\038\001\039\001\040\001\041\001\042\001\043\001\
\044\001\045\001\046\001\047\001\048\001\006\001\029\001\030\001\
\031\001\032\001\033\001\034\001\035\001\036\001\037\001\038\001\
\039\001\040\001\041\001\042\001\043\001\044\001\045\001\046\001\
\047\001\255\255\255\255\255\255\029\001\030\001\031\001\032\001\
\033\001\034\001\035\001\036\001\037\001\038\001\039\001\040\001\
\041\001\042\001\043\001\044\001\045\001\046\001\047\001"

let yynames_const = "\
  SIZEOF\000\
  PTR_OP\000\
  INC_OP\000\
  DEC_OP\000\
  LEFT_OP\000\
  RIGHT_OP\000\
  LE_OP\000\
  GE_OP\000\
  EQ_OP\000\
  NE_OP\000\
  AND_OP\000\
  OR_OP\000\
  MUL_ASSIGN\000\
  DIV_ASSIGN\000\
  MOD_ASSIGN\000\
  ADD_ASSIGN\000\
  SUB_ASSIGN\000\
  LEFT_ASSIGN\000\
  RIGHT_ASSIGN\000\
  AND_ASSIGN\000\
  XOR_ASSIGN\000\
  OR_ASSIGN\000\
  TYPEDEF\000\
  EXTERN\000\
  STATIC\000\
  AUTO\000\
  REGISTER\000\
  CHAR\000\
  SHORT\000\
  INT\000\
  LONG\000\
  SIGNED\000\
  UNSIGNED\000\
  FLOAT\000\
  DOUBLE\000\
  CONST\000\
  VOLATILE\000\
  VOID\000\
  STRUCT\000\
  UNION\000\
  ENUM\000\
  ELLIPSIS\000\
  CASE\000\
  DEFAULT\000\
  IF\000\
  ELSE\000\
  SWITCH\000\
  WHILE\000\
  DO\000\
  FOR\000\
  GOTO\000\
  CONTINUE\000\
  BREAK\000\
  RETURN\000\
  SEMICOLON\000\
  LBRACE\000\
  RBRACE\000\
  COMMA\000\
  COLON\000\
  EQUAL\000\
  LPAR\000\
  RPAR\000\
  LSQUARE\000\
  RSQUARE\000\
  DOT\000\
  AMP\000\
  EXL\000\
  TILDE\000\
  MINUS\000\
  PLUS\000\
  STAR\000\
  SLASH\000\
  PERCENT\000\
  LT\000\
  GT\000\
  HAT\000\
  PIPE\000\
  QUESTION\000\
  EOF\000\
  "

let yynames_block = "\
  ANNOT\000\
  WDECL\000\
  IDENTIFIER\000\
  CONSTANT\000\
  STRING_LITERAL\000\
  TYPE_NAME\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'translation_unit) in
    Obj.repr(
# 228 "c/cparser.mly"
                               ( _1 )
# 1348 "c/cparser.ml"
               : Cast.file))
; (fun parser_env ->
    Obj.repr(
# 229 "c/cparser.mly"
              ( [] )
# 1354 "c/cparser.ml"
               : Cast.file))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 233 "c/cparser.mly"
                     ( locate (CEvar _1) )
# 1361 "c/cparser.ml"
               : 'primary_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 234 "c/cparser.mly"
                   ( locate (CEconstant _1) )
# 1368 "c/cparser.ml"
               : 'primary_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 235 "c/cparser.mly"
                         ( locate (CEstring_literal _1) )
# 1375 "c/cparser.ml"
               : 'primary_expression))
; (fun parser_env ->
    let _2 = (peek_val parser_env 1 : 'expression) in
    Obj.repr(
# 236 "c/cparser.mly"
                               ( _2 )
# 1382 "c/cparser.ml"
               : 'primary_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'primary_expression) in
    Obj.repr(
# 241 "c/cparser.mly"
            ( _1 )
# 1389 "c/cparser.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 3 : 'postfix_expression) in
    let _3 = (peek_val parser_env 1 : 'expression) in
    Obj.repr(
# 243 "c/cparser.mly"
     ( locate (CEarrget (_1, _3)) )
# 1397 "c/cparser.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'postfix_expression) in
    Obj.repr(
# 245 "c/cparser.mly"
     ( locate (CEcall (_1, [])) )
# 1404 "c/cparser.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 3 : 'postfix_expression) in
    let _3 = (peek_val parser_env 1 : 'argument_expression_list) in
    Obj.repr(
# 247 "c/cparser.mly"
     ( locate (CEcall (_1, _3)) )
# 1412 "c/cparser.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'postfix_expression) in
    let _3 = (peek_val parser_env 0 : 'identifier) in
    Obj.repr(
# 249 "c/cparser.mly"
     ( locate (CEdot (_1, _3)) )
# 1420 "c/cparser.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'postfix_expression) in
    let _3 = (peek_val parser_env 0 : 'identifier) in
    Obj.repr(
# 251 "c/cparser.mly"
     ( locate (CEarrow (_1, _3)) )
# 1428 "c/cparser.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'postfix_expression) in
    Obj.repr(
# 253 "c/cparser.mly"
     ( locate (CEunary (Postfix_inc, _1)) )
# 1435 "c/cparser.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'postfix_expression) in
    Obj.repr(
# 255 "c/cparser.mly"
     ( locate (CEunary (Postfix_dec, _1)) )
# 1442 "c/cparser.ml"
               : 'postfix_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'assignment_expression) in
    Obj.repr(
# 259 "c/cparser.mly"
                                ( [_1] )
# 1449 "c/cparser.ml"
               : 'argument_expression_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'argument_expression_list) in
    let _3 = (peek_val parser_env 0 : 'assignment_expression) in
    Obj.repr(
# 260 "c/cparser.mly"
                                                               ( _1 @ [_3] )
# 1457 "c/cparser.ml"
               : 'argument_expression_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'postfix_expression) in
    Obj.repr(
# 264 "c/cparser.mly"
                             ( _1 )
# 1464 "c/cparser.ml"
               : 'unary_expression))
; (fun parser_env ->
    let _2 = (peek_val parser_env 0 : 'unary_expression) in
    Obj.repr(
# 265 "c/cparser.mly"
                                  ( locate (CEunary (Prefix_inc, _2)) )
# 1471 "c/cparser.ml"
               : 'unary_expression))
; (fun parser_env ->
    let _2 = (peek_val parser_env 0 : 'unary_expression) in
    Obj.repr(
# 266 "c/cparser.mly"
                                  ( locate (CEunary (Prefix_dec, _2)) )
# 1478 "c/cparser.ml"
               : 'unary_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'unary_operator) in
    let _2 = (peek_val parser_env 0 : 'cast_expression) in
    Obj.repr(
# 267 "c/cparser.mly"
                                         ( locate (CEunary (_1, _2)) )
# 1486 "c/cparser.ml"
               : 'unary_expression))
; (fun parser_env ->
    let _2 = (peek_val parser_env 0 : 'unary_expression) in
    Obj.repr(
# 268 "c/cparser.mly"
                                  ( locate (CEsizeof_expr _2) )
# 1493 "c/cparser.ml"
               : 'unary_expression))
; (fun parser_env ->
    let _3 = (peek_val parser_env 1 : 'type_name) in
    Obj.repr(
# 270 "c/cparser.mly"
     ( let s,d = _3 in locate (CEsizeof (interp_type s d)) )
# 1500 "c/cparser.ml"
               : 'unary_expression))
; (fun parser_env ->
    Obj.repr(
# 274 "c/cparser.mly"
              ( Uamp )
# 1506 "c/cparser.ml"
               : 'unary_operator))
; (fun parser_env ->
    Obj.repr(
# 275 "c/cparser.mly"
               ( Ustar )
# 1512 "c/cparser.ml"
               : 'unary_operator))
; (fun parser_env ->
    Obj.repr(
# 276 "c/cparser.mly"
               ( Uplus )
# 1518 "c/cparser.ml"
               : 'unary_operator))
; (fun parser_env ->
    Obj.repr(
# 277 "c/cparser.mly"
                ( Uminus )
# 1524 "c/cparser.ml"
               : 'unary_operator))
; (fun parser_env ->
    Obj.repr(
# 278 "c/cparser.mly"
                ( Utilde )
# 1530 "c/cparser.ml"
               : 'unary_operator))
; (fun parser_env ->
    Obj.repr(
# 279 "c/cparser.mly"
              ( Not )
# 1536 "c/cparser.ml"
               : 'unary_operator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'unary_expression) in
    Obj.repr(
# 283 "c/cparser.mly"
                           ( _1 )
# 1543 "c/cparser.ml"
               : 'cast_expression))
; (fun parser_env ->
    let _2 = (peek_val parser_env 2 : 'type_name) in
    let _4 = (peek_val parser_env 0 : 'cast_expression) in
    Obj.repr(
# 285 "c/cparser.mly"
     ( let s,d = _2 in locate (CEcast (interp_type s d, _4)) )
# 1551 "c/cparser.ml"
               : 'cast_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'cast_expression) in
    Obj.repr(
# 290 "c/cparser.mly"
            ( _1 )
# 1558 "c/cparser.ml"
               : 'multiplicative_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'multiplicative_expression) in
    let _3 = (peek_val parser_env 0 : 'cast_expression) in
    Obj.repr(
# 292 "c/cparser.mly"
     ( locate (CEbinary (_1, Mult, _3)) )
# 1566 "c/cparser.ml"
               : 'multiplicative_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'multiplicative_expression) in
    let _3 = (peek_val parser_env 0 : 'cast_expression) in
    Obj.repr(
# 294 "c/cparser.mly"
     ( locate (CEbinary (_1, Div, _3)) )
# 1574 "c/cparser.ml"
               : 'multiplicative_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'multiplicative_expression) in
    let _3 = (peek_val parser_env 0 : 'cast_expression) in
    Obj.repr(
# 296 "c/cparser.mly"
     ( locate (CEbinary (_1, Mod, _3)) )
# 1582 "c/cparser.ml"
               : 'multiplicative_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'multiplicative_expression) in
    Obj.repr(
# 301 "c/cparser.mly"
           ( _1 )
# 1589 "c/cparser.ml"
               : 'additive_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'additive_expression) in
    let _3 = (peek_val parser_env 0 : 'multiplicative_expression) in
    Obj.repr(
# 303 "c/cparser.mly"
     ( locate (CEbinary (_1, Plus, _3)) )
# 1597 "c/cparser.ml"
               : 'additive_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'additive_expression) in
    let _3 = (peek_val parser_env 0 : 'multiplicative_expression) in
    Obj.repr(
# 305 "c/cparser.mly"
     ( locate (CEbinary (_1, Minus, _3)) )
# 1605 "c/cparser.ml"
               : 'additive_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'additive_expression) in
    Obj.repr(
# 309 "c/cparser.mly"
                              ( _1 )
# 1612 "c/cparser.ml"
               : 'shift_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'shift_expression) in
    let _3 = (peek_val parser_env 0 : 'additive_expression) in
    Obj.repr(
# 311 "c/cparser.mly"
     ( locate (CEshift (_1, Left, _3)) )
# 1620 "c/cparser.ml"
               : 'shift_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'shift_expression) in
    let _3 = (peek_val parser_env 0 : 'additive_expression) in
    Obj.repr(
# 313 "c/cparser.mly"
     ( locate (CEshift (_1, Right, _3)) )
# 1628 "c/cparser.ml"
               : 'shift_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'shift_expression) in
    Obj.repr(
# 318 "c/cparser.mly"
            ( _1 )
# 1635 "c/cparser.ml"
               : 'relational_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'relational_expression) in
    let _3 = (peek_val parser_env 0 : 'shift_expression) in
    Obj.repr(
# 320 "c/cparser.mly"
     ( locate (CEbinary (_1, Lt, _3)) )
# 1643 "c/cparser.ml"
               : 'relational_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'relational_expression) in
    let _3 = (peek_val parser_env 0 : 'shift_expression) in
    Obj.repr(
# 322 "c/cparser.mly"
     ( locate (CEbinary (_1, Gt, _3)) )
# 1651 "c/cparser.ml"
               : 'relational_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'relational_expression) in
    let _3 = (peek_val parser_env 0 : 'shift_expression) in
    Obj.repr(
# 324 "c/cparser.mly"
     ( locate (CEbinary (_1, Le, _3)) )
# 1659 "c/cparser.ml"
               : 'relational_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'relational_expression) in
    let _3 = (peek_val parser_env 0 : 'shift_expression) in
    Obj.repr(
# 326 "c/cparser.mly"
     ( locate (CEbinary (_1, Ge, _3)) )
# 1667 "c/cparser.ml"
               : 'relational_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'relational_expression) in
    Obj.repr(
# 331 "c/cparser.mly"
            ( _1 )
# 1674 "c/cparser.ml"
               : 'equality_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'equality_expression) in
    let _3 = (peek_val parser_env 0 : 'relational_expression) in
    Obj.repr(
# 333 "c/cparser.mly"
     ( locate (CEbinary (_1, Eq, _3)) )
# 1682 "c/cparser.ml"
               : 'equality_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'equality_expression) in
    let _3 = (peek_val parser_env 0 : 'relational_expression) in
    Obj.repr(
# 335 "c/cparser.mly"
     ( locate (CEbinary (_1, Neq, _3)) )
# 1690 "c/cparser.ml"
               : 'equality_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'equality_expression) in
    Obj.repr(
# 340 "c/cparser.mly"
            ( _1 )
# 1697 "c/cparser.ml"
               : 'and_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'and_expression) in
    let _3 = (peek_val parser_env 0 : 'equality_expression) in
    Obj.repr(
# 342 "c/cparser.mly"
     ( locate (CEbinary (_1, Bw_and, _3)) )
# 1705 "c/cparser.ml"
               : 'and_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'and_expression) in
    Obj.repr(
# 347 "c/cparser.mly"
            ( _1 )
# 1712 "c/cparser.ml"
               : 'exclusive_or_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'exclusive_or_expression) in
    let _3 = (peek_val parser_env 0 : 'and_expression) in
    Obj.repr(
# 349 "c/cparser.mly"
     ( locate (CEbinary (_1, Bw_xor, _3)) )
# 1720 "c/cparser.ml"
               : 'exclusive_or_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'exclusive_or_expression) in
    Obj.repr(
# 354 "c/cparser.mly"
            ( _1 )
# 1727 "c/cparser.ml"
               : 'inclusive_or_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'inclusive_or_expression) in
    let _3 = (peek_val parser_env 0 : 'exclusive_or_expression) in
    Obj.repr(
# 356 "c/cparser.mly"
     ( locate (CEbinary (_1, Bw_or, _3)) )
# 1735 "c/cparser.ml"
               : 'inclusive_or_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'inclusive_or_expression) in
    Obj.repr(
# 361 "c/cparser.mly"
            ( _1 )
# 1742 "c/cparser.ml"
               : 'logical_and_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'logical_and_expression) in
    let _3 = (peek_val parser_env 0 : 'inclusive_or_expression) in
    Obj.repr(
# 363 "c/cparser.mly"
     ( locate (CEbinary (_1, And, _3)) )
# 1750 "c/cparser.ml"
               : 'logical_and_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'logical_and_expression) in
    Obj.repr(
# 368 "c/cparser.mly"
            ( _1 )
# 1757 "c/cparser.ml"
               : 'logical_or_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'logical_or_expression) in
    let _3 = (peek_val parser_env 0 : 'logical_and_expression) in
    Obj.repr(
# 370 "c/cparser.mly"
     ( locate (CEbinary (_1, Or, _3)) )
# 1765 "c/cparser.ml"
               : 'logical_or_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'logical_or_expression) in
    Obj.repr(
# 375 "c/cparser.mly"
            ( _1 )
# 1772 "c/cparser.ml"
               : 'conditional_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 4 : 'logical_or_expression) in
    let _3 = (peek_val parser_env 2 : 'expression) in
    let _5 = (peek_val parser_env 0 : 'conditional_expression) in
    Obj.repr(
# 377 "c/cparser.mly"
     ( locate (CEcond (_1, _3, _5)) )
# 1781 "c/cparser.ml"
               : 'conditional_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'conditional_expression) in
    Obj.repr(
# 382 "c/cparser.mly"
            ( _1 )
# 1788 "c/cparser.ml"
               : 'assignment_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'unary_expression) in
    let _2 = (peek_val parser_env 1 : 'assignment_operator) in
    let _3 = (peek_val parser_env 0 : 'assignment_expression) in
    Obj.repr(
# 384 "c/cparser.mly"
     ( locate (CEassign (_1, _2, _3)) )
# 1797 "c/cparser.ml"
               : 'assignment_expression))
; (fun parser_env ->
    Obj.repr(
# 388 "c/cparser.mly"
                ( Aequal )
# 1803 "c/cparser.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 389 "c/cparser.mly"
                     ( Amul )
# 1809 "c/cparser.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 390 "c/cparser.mly"
                     ( Adiv )
# 1815 "c/cparser.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 391 "c/cparser.mly"
                     ( Amod )
# 1821 "c/cparser.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 392 "c/cparser.mly"
                     ( Aadd )
# 1827 "c/cparser.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 393 "c/cparser.mly"
                     ( Asub )
# 1833 "c/cparser.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 394 "c/cparser.mly"
                      ( Aleft )
# 1839 "c/cparser.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 395 "c/cparser.mly"
                       ( Aright )
# 1845 "c/cparser.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 396 "c/cparser.mly"
                     ( Aand )
# 1851 "c/cparser.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 397 "c/cparser.mly"
                     ( Axor )
# 1857 "c/cparser.ml"
               : 'assignment_operator))
; (fun parser_env ->
    Obj.repr(
# 398 "c/cparser.mly"
                    ( Aor )
# 1863 "c/cparser.ml"
               : 'assignment_operator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'assignment_expression) in
    Obj.repr(
# 402 "c/cparser.mly"
                                ( _1 )
# 1870 "c/cparser.ml"
               : 'expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'expression) in
    let _3 = (peek_val parser_env 0 : 'assignment_expression) in
    Obj.repr(
# 403 "c/cparser.mly"
                                                 ( locate (CEseq (_1, _3)) )
# 1878 "c/cparser.ml"
               : 'expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'conditional_expression) in
    Obj.repr(
# 407 "c/cparser.mly"
                                 ( _1 )
# 1885 "c/cparser.ml"
               : 'constant_expression))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'declaration_specifiers) in
    Obj.repr(
# 412 "c/cparser.mly"
            ( warning "empty declaration"; [] )
# 1892 "c/cparser.ml"
               : 'declaration))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'declaration_specifiers) in
    let _2 = (peek_val parser_env 1 : 'init_declarator_list) in
    Obj.repr(
# 414 "c/cparser.mly"
     ( List.map locate (declaration _1 _2) )
# 1900 "c/cparser.ml"
               : 'declaration))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : int * string) in
    Obj.repr(
# 416 "c/cparser.mly"
     ( [locate (Cspecdecl _1)] )
# 1907 "c/cparser.ml"
               : 'declaration))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'storage_class_specifier) in
    Obj.repr(
# 421 "c/cparser.mly"
                                              ( [_1] )
# 1914 "c/cparser.ml"
               : 'declaration_specifiers))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'storage_class_specifier) in
    let _2 = (peek_val parser_env 0 : 'declaration_specifiers) in
    Obj.repr(
# 422 "c/cparser.mly"
                                                         ( _1 :: _2 )
# 1922 "c/cparser.ml"
               : 'declaration_specifiers))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'type_specifier) in
    Obj.repr(
# 423 "c/cparser.mly"
                         ( [_1] )
# 1929 "c/cparser.ml"
               : 'declaration_specifiers))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'type_specifier) in
    let _2 = (peek_val parser_env 0 : 'declaration_specifiers_no_name) in
    Obj.repr(
# 424 "c/cparser.mly"
                                                        ( _1 :: _2 )
# 1937 "c/cparser.ml"
               : 'declaration_specifiers))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'type_qualifier) in
    Obj.repr(
# 425 "c/cparser.mly"
                                     ( [_1] )
# 1944 "c/cparser.ml"
               : 'declaration_specifiers))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'type_qualifier) in
    let _2 = (peek_val parser_env 0 : 'declaration_specifiers) in
    Obj.repr(
# 426 "c/cparser.mly"
                                                ( _1 :: _2 )
# 1952 "c/cparser.ml"
               : 'declaration_specifiers))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'storage_class_specifier) in
    Obj.repr(
# 430 "c/cparser.mly"
                                              ( [_1] )
# 1959 "c/cparser.ml"
               : 'declaration_specifiers_no_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'storage_class_specifier) in
    let _2 = (peek_val parser_env 0 : 'declaration_specifiers_no_name) in
    Obj.repr(
# 431 "c/cparser.mly"
                                                                 ( _1 :: _2 )
# 1967 "c/cparser.ml"
               : 'declaration_specifiers_no_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'type_specifier_no_name) in
    Obj.repr(
# 432 "c/cparser.mly"
                                 ( [_1] )
# 1974 "c/cparser.ml"
               : 'declaration_specifiers_no_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'type_specifier_no_name) in
    let _2 = (peek_val parser_env 0 : 'declaration_specifiers_no_name) in
    Obj.repr(
# 433 "c/cparser.mly"
                                                                ( _1 :: _2 )
# 1982 "c/cparser.ml"
               : 'declaration_specifiers_no_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'type_qualifier) in
    Obj.repr(
# 434 "c/cparser.mly"
                                     ( [_1] )
# 1989 "c/cparser.ml"
               : 'declaration_specifiers_no_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'type_qualifier) in
    let _2 = (peek_val parser_env 0 : 'declaration_specifiers) in
    Obj.repr(
# 435 "c/cparser.mly"
                                                ( _1 :: _2 )
# 1997 "c/cparser.ml"
               : 'declaration_specifiers_no_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'init_declarator) in
    Obj.repr(
# 439 "c/cparser.mly"
                          ( [_1] )
# 2004 "c/cparser.ml"
               : 'init_declarator_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'init_declarator_list) in
    let _3 = (peek_val parser_env 0 : 'init_declarator) in
    Obj.repr(
# 440 "c/cparser.mly"
                                                     ( _1 @ [_3] )
# 2012 "c/cparser.ml"
               : 'init_declarator_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'declarator) in
    Obj.repr(
# 445 "c/cparser.mly"
            ( _1, Inothing )
# 2019 "c/cparser.ml"
               : 'init_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'declarator) in
    let _3 = (peek_val parser_env 0 : 'c_initializer) in
    Obj.repr(
# 447 "c/cparser.mly"
     ( _1, _3 )
# 2027 "c/cparser.ml"
               : 'init_declarator))
; (fun parser_env ->
    Obj.repr(
# 451 "c/cparser.mly"
                  ( Stypedef )
# 2033 "c/cparser.ml"
               : 'storage_class_specifier))
; (fun parser_env ->
    Obj.repr(
# 452 "c/cparser.mly"
                 ( Sstorage Extern )
# 2039 "c/cparser.ml"
               : 'storage_class_specifier))
; (fun parser_env ->
    Obj.repr(
# 453 "c/cparser.mly"
                 ( Sstorage Static )
# 2045 "c/cparser.ml"
               : 'storage_class_specifier))
; (fun parser_env ->
    Obj.repr(
# 454 "c/cparser.mly"
               ( Sstorage Auto )
# 2051 "c/cparser.ml"
               : 'storage_class_specifier))
; (fun parser_env ->
    Obj.repr(
# 455 "c/cparser.mly"
                   ( Sstorage Register )
# 2057 "c/cparser.ml"
               : 'storage_class_specifier))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'type_specifier_no_name) in
    Obj.repr(
# 459 "c/cparser.mly"
                                 ( _1 )
# 2064 "c/cparser.ml"
               : 'type_specifier))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 460 "c/cparser.mly"
                    ( Stype (CTvar _1) )
# 2071 "c/cparser.ml"
               : 'type_specifier))
; (fun parser_env ->
    Obj.repr(
# 463 "c/cparser.mly"
               ( Stype CTvoid )
# 2077 "c/cparser.ml"
               : 'type_specifier_no_name))
; (fun parser_env ->
    Obj.repr(
# 464 "c/cparser.mly"
               ( Stype CTchar )
# 2083 "c/cparser.ml"
               : 'type_specifier_no_name))
; (fun parser_env ->
    Obj.repr(
# 465 "c/cparser.mly"
                ( Sshort )
# 2089 "c/cparser.ml"
               : 'type_specifier_no_name))
; (fun parser_env ->
    Obj.repr(
# 466 "c/cparser.mly"
              ( Stype CTint )
# 2095 "c/cparser.ml"
               : 'type_specifier_no_name))
; (fun parser_env ->
    Obj.repr(
# 467 "c/cparser.mly"
               ( Slong )
# 2101 "c/cparser.ml"
               : 'type_specifier_no_name))
; (fun parser_env ->
    Obj.repr(
# 468 "c/cparser.mly"
                ( Stype CTfloat )
# 2107 "c/cparser.ml"
               : 'type_specifier_no_name))
; (fun parser_env ->
    Obj.repr(
# 469 "c/cparser.mly"
                 ( Stype CTdouble )
# 2113 "c/cparser.ml"
               : 'type_specifier_no_name))
; (fun parser_env ->
    Obj.repr(
# 470 "c/cparser.mly"
                 ( Ssigned true )
# 2119 "c/cparser.ml"
               : 'type_specifier_no_name))
; (fun parser_env ->
    Obj.repr(
# 471 "c/cparser.mly"
                   ( Ssigned false )
# 2125 "c/cparser.ml"
               : 'type_specifier_no_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'struct_or_union_specifier) in
    Obj.repr(
# 472 "c/cparser.mly"
                                    ( _1 )
# 2132 "c/cparser.ml"
               : 'type_specifier_no_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'enum_specifier) in
    Obj.repr(
# 473 "c/cparser.mly"
                         ( _1 )
# 2139 "c/cparser.ml"
               : 'type_specifier_no_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 477 "c/cparser.mly"
                     ( _1 )
# 2146 "c/cparser.ml"
               : 'identifier))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 478 "c/cparser.mly"
                     ( _1 )
# 2153 "c/cparser.ml"
               : 'identifier))
; (fun parser_env ->
    let _1 = (peek_val parser_env 4 : 'struct_or_union) in
    let _2 = (peek_val parser_env 3 : 'identifier) in
    let _4 = (peek_val parser_env 1 : 'struct_declaration_list) in
    Obj.repr(
# 483 "c/cparser.mly"
            ( if _1 then 
		Sstruct_decl (Some _2, _4) 
	      else 
		Sunion_decl (Some _2, _4) )
# 2165 "c/cparser.ml"
               : 'struct_or_union_specifier))
; (fun parser_env ->
    let _1 = (peek_val parser_env 3 : 'struct_or_union) in
    let _3 = (peek_val parser_env 1 : 'struct_declaration_list) in
    Obj.repr(
# 488 "c/cparser.mly"
     ( if _1 then Sstruct_decl (None, _3) else Sunion_decl (None, _3) )
# 2173 "c/cparser.ml"
               : 'struct_or_union_specifier))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'struct_or_union) in
    let _2 = (peek_val parser_env 0 : 'identifier) in
    Obj.repr(
# 490 "c/cparser.mly"
     ( Stype (if _1 then CTstruct_named _2 else CTunion_named _2) )
# 2181 "c/cparser.ml"
               : 'struct_or_union_specifier))
; (fun parser_env ->
    Obj.repr(
# 494 "c/cparser.mly"
                 ( true )
# 2187 "c/cparser.ml"
               : 'struct_or_union))
; (fun parser_env ->
    Obj.repr(
# 495 "c/cparser.mly"
                ( false )
# 2193 "c/cparser.ml"
               : 'struct_or_union))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'struct_declaration) in
    Obj.repr(
# 499 "c/cparser.mly"
                             ( _1 )
# 2200 "c/cparser.ml"
               : 'struct_declaration_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'struct_declaration_list) in
    let _2 = (peek_val parser_env 0 : 'struct_declaration) in
    Obj.repr(
# 500 "c/cparser.mly"
                                                     ( _1 @ _2 )
# 2208 "c/cparser.ml"
               : 'struct_declaration_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'specifier_qualifier_list) in
    let _2 = (peek_val parser_env 1 : 'struct_declarator_list) in
    Obj.repr(
# 505 "c/cparser.mly"
            ( let s = _1 in List.map (fun (id,d) -> s,d,id) _2 )
# 2216 "c/cparser.ml"
               : 'struct_declaration))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'type_specifier) in
    let _2 = (peek_val parser_env 0 : 'specifier_qualifier_list_no_name) in
    Obj.repr(
# 509 "c/cparser.mly"
                                                          ( _1 :: _2 )
# 2224 "c/cparser.ml"
               : 'specifier_qualifier_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'type_specifier) in
    Obj.repr(
# 510 "c/cparser.mly"
                         ( [_1] )
# 2231 "c/cparser.ml"
               : 'specifier_qualifier_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'type_qualifier) in
    let _2 = (peek_val parser_env 0 : 'specifier_qualifier_list) in
    Obj.repr(
# 511 "c/cparser.mly"
                                                  ( _1 :: _2 )
# 2239 "c/cparser.ml"
               : 'specifier_qualifier_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'type_qualifier) in
    Obj.repr(
# 512 "c/cparser.mly"
                                     ( [_1] )
# 2246 "c/cparser.ml"
               : 'specifier_qualifier_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'type_specifier_no_name) in
    let _2 = (peek_val parser_env 0 : 'specifier_qualifier_list_no_name) in
    Obj.repr(
# 516 "c/cparser.mly"
                                                                  ( _1 :: _2 )
# 2254 "c/cparser.ml"
               : 'specifier_qualifier_list_no_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'type_specifier_no_name) in
    Obj.repr(
# 517 "c/cparser.mly"
                                 ( [_1] )
# 2261 "c/cparser.ml"
               : 'specifier_qualifier_list_no_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'type_qualifier) in
    let _2 = (peek_val parser_env 0 : 'specifier_qualifier_list_no_name) in
    Obj.repr(
# 518 "c/cparser.mly"
                                                          ( _1 :: _2 )
# 2269 "c/cparser.ml"
               : 'specifier_qualifier_list_no_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'type_qualifier) in
    Obj.repr(
# 519 "c/cparser.mly"
                         ( [_1] )
# 2276 "c/cparser.ml"
               : 'specifier_qualifier_list_no_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'struct_declarator) in
    Obj.repr(
# 523 "c/cparser.mly"
                            ( [_1] )
# 2283 "c/cparser.ml"
               : 'struct_declarator_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'struct_declarator_list) in
    let _3 = (peek_val parser_env 0 : 'struct_declarator) in
    Obj.repr(
# 524 "c/cparser.mly"
                                                         ( _1 @ [_3] )
# 2291 "c/cparser.ml"
               : 'struct_declarator_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'declarator) in
    Obj.repr(
# 529 "c/cparser.mly"
            ( _1 )
# 2298 "c/cparser.ml"
               : 'struct_declarator))
; (fun parser_env ->
    let _2 = (peek_val parser_env 0 : 'constant_expression) in
    Obj.repr(
# 531 "c/cparser.mly"
     ( "_", Dbitfield (Dsimple, _2) )
# 2305 "c/cparser.ml"
               : 'struct_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'declarator) in
    let _3 = (peek_val parser_env 0 : 'constant_expression) in
    Obj.repr(
# 533 "c/cparser.mly"
     ( let x,d = _1 in x, Dbitfield (d, _3) )
# 2313 "c/cparser.ml"
               : 'struct_declarator))
; (fun parser_env ->
    let _3 = (peek_val parser_env 1 : 'enumerator_list) in
    Obj.repr(
# 538 "c/cparser.mly"
            ( Stype (CTenum_decl (None, _3)) )
# 2320 "c/cparser.ml"
               : 'enum_specifier))
; (fun parser_env ->
    let _2 = (peek_val parser_env 3 : 'identifier) in
    let _4 = (peek_val parser_env 1 : 'enumerator_list) in
    Obj.repr(
# 540 "c/cparser.mly"
     ( Stype (CTenum_decl (Some _2, _4)) )
# 2328 "c/cparser.ml"
               : 'enum_specifier))
; (fun parser_env ->
    let _2 = (peek_val parser_env 0 : 'identifier) in
    Obj.repr(
# 542 "c/cparser.mly"
     ( Stype (CTenum_named _2) )
# 2335 "c/cparser.ml"
               : 'enum_specifier))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'enumerator) in
    Obj.repr(
# 546 "c/cparser.mly"
                     ( [_1] )
# 2342 "c/cparser.ml"
               : 'enumerator_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'enumerator_list) in
    let _3 = (peek_val parser_env 0 : 'enumerator) in
    Obj.repr(
# 547 "c/cparser.mly"
                                           ( _1 @ [_3] )
# 2350 "c/cparser.ml"
               : 'enumerator_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 551 "c/cparser.mly"
                     ( _1, None )
# 2357 "c/cparser.ml"
               : 'enumerator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : string) in
    let _3 = (peek_val parser_env 0 : 'constant_expression) in
    Obj.repr(
# 552 "c/cparser.mly"
                                               ( _1, Some _3 )
# 2365 "c/cparser.ml"
               : 'enumerator))
; (fun parser_env ->
    Obj.repr(
# 556 "c/cparser.mly"
                ( Sconst )
# 2371 "c/cparser.ml"
               : 'type_qualifier))
; (fun parser_env ->
    Obj.repr(
# 557 "c/cparser.mly"
                   ( Svolatile )
# 2377 "c/cparser.ml"
               : 'type_qualifier))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'pointer) in
    let _2 = (peek_val parser_env 0 : 'direct_declarator) in
    Obj.repr(
# 561 "c/cparser.mly"
                                    ( let id,d = _2 in id, _1 d )
# 2385 "c/cparser.ml"
               : 'declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'direct_declarator) in
    Obj.repr(
# 562 "c/cparser.mly"
                            ( _1 )
# 2392 "c/cparser.ml"
               : 'declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'identifier) in
    Obj.repr(
# 567 "c/cparser.mly"
            ( _1, Dsimple )
# 2399 "c/cparser.ml"
               : 'direct_declarator))
; (fun parser_env ->
    let _2 = (peek_val parser_env 1 : 'declarator) in
    Obj.repr(
# 569 "c/cparser.mly"
     ( uns() )
# 2406 "c/cparser.ml"
               : 'direct_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 3 : 'direct_declarator) in
    let _3 = (peek_val parser_env 1 : 'constant_expression) in
    Obj.repr(
# 571 "c/cparser.mly"
     ( let id,d = _1 in id, Darray (d, Some _3) )
# 2414 "c/cparser.ml"
               : 'direct_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'direct_declarator) in
    Obj.repr(
# 573 "c/cparser.mly"
     ( let id,d = _1 in id, Darray (d, None) )
# 2421 "c/cparser.ml"
               : 'direct_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 4 : 'direct_declarator) in
    let _3 = (peek_val parser_env 2 : 'parameter_type_list) in
    let _5 = (peek_val parser_env 0 : 'annot) in
    Obj.repr(
# 575 "c/cparser.mly"
     ( let id,d = _1 in id, Dfunction (d, _3, _5) )
# 2430 "c/cparser.ml"
               : 'direct_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 3 : 'direct_declarator) in
    let _3 = (peek_val parser_env 1 : 'identifier_list) in
    Obj.repr(
# 577 "c/cparser.mly"
     ( uns() )
# 2438 "c/cparser.ml"
               : 'direct_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 3 : 'direct_declarator) in
    let _4 = (peek_val parser_env 0 : 'annot) in
    Obj.repr(
# 579 "c/cparser.mly"
            ( let id,d = _1 in id, Dfunction (d, [], _4) )
# 2446 "c/cparser.ml"
               : 'direct_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : int * string) in
    Obj.repr(
# 584 "c/cparser.mly"
                        ( Some _1 )
# 2453 "c/cparser.ml"
               : 'annot))
; (fun parser_env ->
    Obj.repr(
# 585 "c/cparser.mly"
                        ( None )
# 2459 "c/cparser.ml"
               : 'annot))
; (fun parser_env ->
    Obj.repr(
# 589 "c/cparser.mly"
               ( fun d -> Dpointer d )
# 2465 "c/cparser.ml"
               : 'pointer))
; (fun parser_env ->
    let _2 = (peek_val parser_env 0 : 'type_qualifier_list) in
    Obj.repr(
# 590 "c/cparser.mly"
                                   ( uns () )
# 2472 "c/cparser.ml"
               : 'pointer))
; (fun parser_env ->
    let _2 = (peek_val parser_env 0 : 'pointer) in
    Obj.repr(
# 591 "c/cparser.mly"
                       ( fun d -> Dpointer (_2 d) )
# 2479 "c/cparser.ml"
               : 'pointer))
; (fun parser_env ->
    let _2 = (peek_val parser_env 1 : 'type_qualifier_list) in
    let _3 = (peek_val parser_env 0 : 'pointer) in
    Obj.repr(
# 592 "c/cparser.mly"
                                           ( uns () )
# 2487 "c/cparser.ml"
               : 'pointer))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'type_qualifier) in
    Obj.repr(
# 596 "c/cparser.mly"
                         ( [_1] )
# 2494 "c/cparser.ml"
               : 'type_qualifier_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'type_qualifier_list) in
    let _2 = (peek_val parser_env 0 : 'type_qualifier) in
    Obj.repr(
# 597 "c/cparser.mly"
                                             ( _1 @ [_2] )
# 2502 "c/cparser.ml"
               : 'type_qualifier_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'parameter_list) in
    Obj.repr(
# 602 "c/cparser.mly"
                         ( _1 )
# 2509 "c/cparser.ml"
               : 'parameter_type_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'parameter_list) in
    Obj.repr(
# 603 "c/cparser.mly"
                                        ( uns() )
# 2516 "c/cparser.ml"
               : 'parameter_type_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'parameter_declaration) in
    Obj.repr(
# 607 "c/cparser.mly"
                                ( [_1] )
# 2523 "c/cparser.ml"
               : 'parameter_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'parameter_list) in
    let _3 = (peek_val parser_env 0 : 'parameter_declaration) in
    Obj.repr(
# 608 "c/cparser.mly"
                                                     ( _1 @ [_3] )
# 2531 "c/cparser.ml"
               : 'parameter_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'declaration_specifiers) in
    let _2 = (peek_val parser_env 0 : 'declarator) in
    Obj.repr(
# 613 "c/cparser.mly"
            ( let id,d = _2 in _1, d, id )
# 2539 "c/cparser.ml"
               : 'parameter_declaration))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'declaration_specifiers) in
    let _2 = (peek_val parser_env 0 : 'abstract_declarator) in
    Obj.repr(
# 615 "c/cparser.mly"
     ( _1, _2, "_" )
# 2547 "c/cparser.ml"
               : 'parameter_declaration))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'declaration_specifiers) in
    Obj.repr(
# 617 "c/cparser.mly"
     ( (_1, Dsimple, "_") )
# 2554 "c/cparser.ml"
               : 'parameter_declaration))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 621 "c/cparser.mly"
                     ( )
# 2561 "c/cparser.ml"
               : 'identifier_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'identifier_list) in
    let _3 = (peek_val parser_env 0 : string) in
    Obj.repr(
# 622 "c/cparser.mly"
                                           ( )
# 2569 "c/cparser.ml"
               : 'identifier_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'specifier_qualifier_list) in
    Obj.repr(
# 626 "c/cparser.mly"
                                   ( _1, Dsimple )
# 2576 "c/cparser.ml"
               : 'type_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'specifier_qualifier_list) in
    let _2 = (peek_val parser_env 0 : 'abstract_declarator) in
    Obj.repr(
# 627 "c/cparser.mly"
                                                       ( _1, _2 )
# 2584 "c/cparser.ml"
               : 'type_name))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'pointer) in
    Obj.repr(
# 631 "c/cparser.mly"
                  ( _1 Dsimple )
# 2591 "c/cparser.ml"
               : 'abstract_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'direct_abstract_declarator) in
    Obj.repr(
# 632 "c/cparser.mly"
                                     ( _1 )
# 2598 "c/cparser.ml"
               : 'abstract_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'pointer) in
    let _2 = (peek_val parser_env 0 : 'direct_abstract_declarator) in
    Obj.repr(
# 633 "c/cparser.mly"
                                             ( _1 _2 )
# 2606 "c/cparser.ml"
               : 'abstract_declarator))
; (fun parser_env ->
    let _2 = (peek_val parser_env 1 : 'abstract_declarator) in
    Obj.repr(
# 638 "c/cparser.mly"
            ( uns () )
# 2613 "c/cparser.ml"
               : 'direct_abstract_declarator))
; (fun parser_env ->
    Obj.repr(
# 640 "c/cparser.mly"
     ( Darray (Dsimple, None) )
# 2619 "c/cparser.ml"
               : 'direct_abstract_declarator))
; (fun parser_env ->
    let _2 = (peek_val parser_env 1 : 'constant_expression) in
    Obj.repr(
# 642 "c/cparser.mly"
     ( Darray (Dsimple, Some _2) )
# 2626 "c/cparser.ml"
               : 'direct_abstract_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'direct_abstract_declarator) in
    Obj.repr(
# 644 "c/cparser.mly"
     ( Darray (_1, None) )
# 2633 "c/cparser.ml"
               : 'direct_abstract_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 3 : 'direct_abstract_declarator) in
    let _3 = (peek_val parser_env 1 : 'constant_expression) in
    Obj.repr(
# 646 "c/cparser.mly"
     ( Darray (_1, Some _3) )
# 2641 "c/cparser.ml"
               : 'direct_abstract_declarator))
; (fun parser_env ->
    Obj.repr(
# 648 "c/cparser.mly"
     ( Dfunction (Dsimple, [], None) )
# 2647 "c/cparser.ml"
               : 'direct_abstract_declarator))
; (fun parser_env ->
    let _2 = (peek_val parser_env 1 : 'parameter_type_list) in
    Obj.repr(
# 650 "c/cparser.mly"
     ( Dfunction (Dsimple, _2, None) )
# 2654 "c/cparser.ml"
               : 'direct_abstract_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'direct_abstract_declarator) in
    Obj.repr(
# 652 "c/cparser.mly"
     ( Dfunction (_1, [], None) )
# 2661 "c/cparser.ml"
               : 'direct_abstract_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 3 : 'direct_abstract_declarator) in
    let _3 = (peek_val parser_env 1 : 'parameter_type_list) in
    Obj.repr(
# 654 "c/cparser.mly"
     ( Dfunction (_1, _3, None) )
# 2669 "c/cparser.ml"
               : 'direct_abstract_declarator))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'assignment_expression) in
    Obj.repr(
# 658 "c/cparser.mly"
                                ( Iexpr _1 )
# 2676 "c/cparser.ml"
               : 'c_initializer))
; (fun parser_env ->
    let _2 = (peek_val parser_env 1 : 'c_initializer_list) in
    Obj.repr(
# 659 "c/cparser.mly"
                                           ( Ilist _2 )
# 2683 "c/cparser.ml"
               : 'c_initializer))
; (fun parser_env ->
    let _2 = (peek_val parser_env 2 : 'c_initializer_list) in
    Obj.repr(
# 660 "c/cparser.mly"
                                                 ( Ilist _2 )
# 2690 "c/cparser.ml"
               : 'c_initializer))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'c_initializer) in
    Obj.repr(
# 664 "c/cparser.mly"
                        ( [_1] )
# 2697 "c/cparser.ml"
               : 'c_initializer_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'c_initializer_list) in
    let _3 = (peek_val parser_env 0 : 'c_initializer) in
    Obj.repr(
# 665 "c/cparser.mly"
                                                 ( _1 @ [_3] )
# 2705 "c/cparser.ml"
               : 'c_initializer_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'labeled_statement) in
    Obj.repr(
# 669 "c/cparser.mly"
                            ( _1 )
# 2712 "c/cparser.ml"
               : 'statement))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'compound_statement) in
    Obj.repr(
# 670 "c/cparser.mly"
                             ( locate (CSblock _1) )
# 2719 "c/cparser.ml"
               : 'statement))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'expression_statement) in
    Obj.repr(
# 671 "c/cparser.mly"
                               ( _1 )
# 2726 "c/cparser.ml"
               : 'statement))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'selection_statement) in
    Obj.repr(
# 672 "c/cparser.mly"
                              ( _1 )
# 2733 "c/cparser.ml"
               : 'statement))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'iteration_statement) in
    Obj.repr(
# 673 "c/cparser.mly"
                              ( _1 )
# 2740 "c/cparser.ml"
               : 'statement))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'jump_statement) in
    Obj.repr(
# 674 "c/cparser.mly"
                         ( _1 )
# 2747 "c/cparser.ml"
               : 'statement))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'identifier) in
    let _3 = (peek_val parser_env 0 : 'statement) in
    Obj.repr(
# 678 "c/cparser.mly"
                                            ( locate (CSlabel (_1, _3)) )
# 2755 "c/cparser.ml"
               : 'labeled_statement))
; (fun parser_env ->
    let _2 = (peek_val parser_env 2 : 'constant_expression) in
    let _4 = (peek_val parser_env 0 : 'statement) in
    Obj.repr(
# 679 "c/cparser.mly"
                                                   ( locate (CScase (_2, _4)) )
# 2763 "c/cparser.ml"
               : 'labeled_statement))
; (fun parser_env ->
    let _3 = (peek_val parser_env 0 : 'statement) in
    Obj.repr(
# 680 "c/cparser.mly"
                                  ( locate (CSdefault _3) )
# 2770 "c/cparser.ml"
               : 'labeled_statement))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'compound_statement_LBRACE) in
    Obj.repr(
# 685 "c/cparser.mly"
            ( Ctypes.pop (); [], [] )
# 2777 "c/cparser.ml"
               : 'compound_statement))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'compound_statement_LBRACE) in
    let _2 = (peek_val parser_env 1 : 'statement_list) in
    Obj.repr(
# 687 "c/cparser.mly"
     ( Ctypes.pop (); [], _2 )
# 2785 "c/cparser.ml"
               : 'compound_statement))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'compound_statement_LBRACE) in
    let _2 = (peek_val parser_env 1 : 'declaration_list) in
    Obj.repr(
# 689 "c/cparser.mly"
     ( Ctypes.pop (); _2, [] )
# 2793 "c/cparser.ml"
               : 'compound_statement))
; (fun parser_env ->
    let _1 = (peek_val parser_env 3 : 'compound_statement_LBRACE) in
    let _2 = (peek_val parser_env 2 : 'declaration_list) in
    let _3 = (peek_val parser_env 1 : 'statement_list) in
    Obj.repr(
# 691 "c/cparser.mly"
     ( Ctypes.pop (); _2, _3 )
# 2802 "c/cparser.ml"
               : 'compound_statement))
; (fun parser_env ->
    Obj.repr(
# 695 "c/cparser.mly"
         ( Ctypes.push () )
# 2808 "c/cparser.ml"
               : 'compound_statement_LBRACE))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'compound_statement) in
    let _2 = (peek_val parser_env 0 : 'annot) in
    Obj.repr(
# 700 "c/cparser.mly"
                                   ( (_1, _2) )
# 2816 "c/cparser.ml"
               : 'compound_statement_with_post))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'declaration) in
    Obj.repr(
# 704 "c/cparser.mly"
                      ( _1 )
# 2823 "c/cparser.ml"
               : 'declaration_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'declaration_list) in
    let _2 = (peek_val parser_env 0 : 'declaration) in
    Obj.repr(
# 705 "c/cparser.mly"
                                       ( _1 @ _2 )
# 2831 "c/cparser.ml"
               : 'declaration_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'statement) in
    Obj.repr(
# 709 "c/cparser.mly"
                    ( [_1] )
# 2838 "c/cparser.ml"
               : 'statement_list))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'statement_list) in
    let _2 = (peek_val parser_env 0 : 'statement) in
    Obj.repr(
# 710 "c/cparser.mly"
                                   ( _1 @ [_2] )
# 2846 "c/cparser.ml"
               : 'statement_list))
; (fun parser_env ->
    Obj.repr(
# 714 "c/cparser.mly"
                    ( locate CSnop )
# 2852 "c/cparser.ml"
               : 'expression_statement))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : int * string) in
    Obj.repr(
# 715 "c/cparser.mly"
                   ( locate (CSannot _1) )
# 2859 "c/cparser.ml"
               : 'expression_statement))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'expression) in
    Obj.repr(
# 716 "c/cparser.mly"
                               ( locate (CSexpr _1) )
# 2866 "c/cparser.ml"
               : 'expression_statement))
; (fun parser_env ->
    let _3 = (peek_val parser_env 2 : 'expression) in
    let _5 = (peek_val parser_env 0 : 'statement) in
    Obj.repr(
# 721 "c/cparser.mly"
            ( locate (CScond (_3, _5, locate CSnop)) )
# 2874 "c/cparser.ml"
               : 'selection_statement))
; (fun parser_env ->
    let _3 = (peek_val parser_env 4 : 'expression) in
    let _5 = (peek_val parser_env 2 : 'statement) in
    let _7 = (peek_val parser_env 0 : 'statement) in
    Obj.repr(
# 723 "c/cparser.mly"
     ( locate (CScond (_3, _5, _7)) )
# 2883 "c/cparser.ml"
               : 'selection_statement))
; (fun parser_env ->
    let _3 = (peek_val parser_env 2 : 'expression) in
    let _5 = (peek_val parser_env 0 : 'statement) in
    Obj.repr(
# 725 "c/cparser.mly"
     ( locate (CSswitch (_3, _5)) )
# 2891 "c/cparser.ml"
               : 'selection_statement))
; (fun parser_env ->
    let _3 = (peek_val parser_env 3 : 'expression) in
    let _5 = (peek_val parser_env 1 : int * string) in
    let _6 = (peek_val parser_env 0 : 'statement) in
    Obj.repr(
# 730 "c/cparser.mly"
            ( locate (CSwhile (_3, _5, _6)) )
# 2900 "c/cparser.ml"
               : 'iteration_statement))
; (fun parser_env ->
    let _2 = (peek_val parser_env 6 : 'statement) in
    let _3 = (peek_val parser_env 5 : int * string) in
    let _6 = (peek_val parser_env 2 : 'expression) in
    Obj.repr(
# 732 "c/cparser.mly"
     ( locate (CSdowhile (_2, _3, _6)) )
# 2909 "c/cparser.ml"
               : 'iteration_statement))
; (fun parser_env ->
    let _3 = (peek_val parser_env 4 : 'expression_statement) in
    let _4 = (peek_val parser_env 3 : 'expression_statement) in
    let _6 = (peek_val parser_env 1 : int * string) in
    let _7 = (peek_val parser_env 0 : 'statement) in
    Obj.repr(
# 735 "c/cparser.mly"
     ( locate (CSfor (expr_of_statement _3, expr_of_statement _4, 
			     None, _6, _7)) )
# 2920 "c/cparser.ml"
               : 'iteration_statement))
; (fun parser_env ->
    let _3 = (peek_val parser_env 5 : 'expression_statement) in
    let _4 = (peek_val parser_env 4 : 'expression_statement) in
    let _5 = (peek_val parser_env 3 : 'expression) in
    let _7 = (peek_val parser_env 1 : int * string) in
    let _8 = (peek_val parser_env 0 : 'statement) in
    Obj.repr(
# 739 "c/cparser.mly"
     ( locate (CSfor (expr_of_statement _3, expr_of_statement _4, 
			     Some _5, _7, _8)) )
# 2932 "c/cparser.ml"
               : 'iteration_statement))
; (fun parser_env ->
    let _2 = (peek_val parser_env 1 : 'identifier) in
    Obj.repr(
# 744 "c/cparser.mly"
                                           ( locate (CSgoto _2) )
# 2939 "c/cparser.ml"
               : 'jump_statement))
; (fun parser_env ->
    Obj.repr(
# 745 "c/cparser.mly"
                             ( locate CScontinue )
# 2945 "c/cparser.ml"
               : 'jump_statement))
; (fun parser_env ->
    Obj.repr(
# 746 "c/cparser.mly"
                          ( locate CSbreak )
# 2951 "c/cparser.ml"
               : 'jump_statement))
; (fun parser_env ->
    Obj.repr(
# 747 "c/cparser.mly"
                           ( locate (CSreturn None) )
# 2957 "c/cparser.ml"
               : 'jump_statement))
; (fun parser_env ->
    let _2 = (peek_val parser_env 1 : 'expression) in
    Obj.repr(
# 748 "c/cparser.mly"
                                      ( locate (CSreturn (Some _2)) )
# 2964 "c/cparser.ml"
               : 'jump_statement))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'external_declaration) in
    Obj.repr(
# 752 "c/cparser.mly"
                               ( _1 )
# 2971 "c/cparser.ml"
               : 'translation_unit))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'translation_unit) in
    let _2 = (peek_val parser_env 0 : 'external_declaration) in
    Obj.repr(
# 753 "c/cparser.mly"
                                                ( _1 @ _2 )
# 2979 "c/cparser.ml"
               : 'translation_unit))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'function_definition) in
    Obj.repr(
# 757 "c/cparser.mly"
                              ( [_1] )
# 2986 "c/cparser.ml"
               : 'external_declaration))
; (fun parser_env ->
    let _1 = (peek_val parser_env 0 : 'declaration) in
    Obj.repr(
# 758 "c/cparser.mly"
                      ( _1 )
# 2993 "c/cparser.ml"
               : 'external_declaration))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'function_prototype) in
    let _2 = (peek_val parser_env 0 : 'compound_statement_with_post) in
    Obj.repr(
# 763 "c/cparser.mly"
            ( Ctypes.pop (); (* pushed by function_prototype *)
	      let b,q = _2 in
	      let ty,id,pl,p = _1 in
	      let l = add_pre_loc (loc_i 2) p in
	      locate (Cfundef (ty, id, pl, with_loc l (p,b,q))) )
# 3005 "c/cparser.ml"
               : 'function_definition))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'declaration_specifiers) in
    let _2 = (peek_val parser_env 1 : 'declarator) in
    let _3 = (peek_val parser_env 0 : 'declaration_list) in
    Obj.repr(
# 772 "c/cparser.mly"
            ( Ctypes.push ();
	      match _2 with
		| id, Dfunction (d, pl, p) ->
		    let ty = interp_type _1 d in
		    let pl = interp_params pl in
		    List.iter (fun (_,x) -> Ctypes.remove x) pl;
		    ty, id, pl, p
		| _ -> uns () )
# 3021 "c/cparser.ml"
               : 'function_prototype))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'declaration_specifiers) in
    let _2 = (peek_val parser_env 0 : 'declarator) in
    Obj.repr(
# 781 "c/cparser.mly"
     ( Ctypes.push ();
	      match _2 with
		| id, Dfunction (d, pl, p) -> 
		    let ty = interp_type _1 d in
		    let pl = interp_params pl in
		    List.iter (fun (_,x) -> Ctypes.remove x) pl;
		    ty, id, pl, p
		| _ -> uns () )
# 3036 "c/cparser.ml"
               : 'function_prototype))
; (fun parser_env ->
    let _1 = (peek_val parser_env 2 : 'declarator) in
    let _2 = (peek_val parser_env 1 : 'declaration_list) in
    let _3 = (peek_val parser_env 0 : 'compound_statement) in
    Obj.repr(
# 790 "c/cparser.mly"
     ( Ctypes.push ();
	      uns () )
# 3046 "c/cparser.ml"
               : 'function_prototype))
; (fun parser_env ->
    let _1 = (peek_val parser_env 1 : 'declarator) in
    let _2 = (peek_val parser_env 0 : 'compound_statement) in
    Obj.repr(
# 793 "c/cparser.mly"
     ( Ctypes.push ();
	      uns () )
# 3055 "c/cparser.ml"
               : 'function_prototype))
(* Entry file *)
; (fun parser_env -> raise (YYexit (peek_val parser_env 0)))
|]
let yytables =
  { actions=yyact;
    transl_const=yytransl_const;
    transl_block=yytransl_block;
    lhs=yylhs;
    len=yylen;
    defred=yydefred;
    dgoto=yydgoto;
    sindex=yysindex;
    rindex=yyrindex;
    gindex=yygindex;
    tablesize=yytablesize;
    table=yytable;
    check=yycheck;
    error_function=parse_error;
    names_const=yynames_const;
    names_block=yynames_block }
let file (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (yyparse yytables 1 lexfun lexbuf : Cast.file)
;;
# 798 "c/cparser.mly"

(* C annotations *)

(***
(* Entries for the C parser *)

type 'a c_parser = int -> string -> 'a

val parse_c_spec : (lexpr asst list * Effect.t * lexpr post option) c_parser
val parse_c_pre : (lexpr asst option * variant option) c_parser
val parse_c_post : (lexpr post option) c_parser
val parse_c_loop_annot : (lexpr asst option * variant) c_parser
val parse_c_decl : decl c_parser
val parse_c_annot : block_st c_parser
***)

(***

let c_pre_condition = gec "c_pre_condition"
let c_post_condition = gec "c_post_condition"
let c_spec = gec "c_spec"
let c_loop_annot = gec "c_loop_annot"
let c_pre = gec "c_pre"
let c_variant = gec "c_variant"
let c_post = gec "c_post"
let c_annot = gec "c_annot"

EXTEND
  c_pre_condition:
  [ [ LIDENT "pre"; p = pre_condition -> p ] ];
  c_post_condition:
  [ [ LIDENT "post"; q = post_condition -> q ] ];
  c_spec:
  [ [ p = OPT c_pre_condition; e = effects; q = OPT c_post_condition; EOI -> 
      (list_of_some p, e, q)
  ] ];
  c_loop_annot:
  [ [ i = OPT invariant; LIDENT "variant"; v = variant -> i,v
  ] ];
  c_pre:
  [ [ p = OPT pre_condition; v = OPT c_variant -> p,v ] ];
  c_variant:
  [ [ LIDENT "variant"; v = variant -> v ] ];
  c_post:
  [ [ q = OPT post_condition -> q ] ];
  c_annot:
  [ [ LIDENT "assert"; p = assertion -> Sassert p 
    | LIDENT "label"; s = ident -> Slabel (Ident.string s) ] ];
END
;;

type 'a c_parser = int -> string -> 'a

let parse_with_offset f n s =
  (* Options.if_debug_3 Format.eprintf "parse_with_offset : %d %s@\n" n s; *)
  let st = Stream.of_string s in
  with_offset n (Grammar.Entry.parse f) st

let parse_c_spec = parse_with_offset c_spec
let parse_c_pre = parse_with_offset c_pre
let parse_c_post = parse_with_offset c_post
let parse_c_loop_annot = parse_with_offset c_loop_annot
let parse_c_decl = parse_with_offset decl
let parse_c_annot = parse_with_offset c_annot

***)

# 3148 "c/cparser.ml"
